pro bin_noaa_daily
;=====================================================================
;@***h*  MW_Sounder_FCDR/bin_noaa_daily.pro
;
; NAME
;        bin_noaa_daily.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For the specified polar orbiter and AMSU channel and range
;              of Julian days, AMSU values are averaged on a 2.5 degree grid.
;              daily gridded values are written to ASCII files for later
;              use.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:     
;              Daily values of AMSU are read from the files:
;
;                 $InPath/YYYYMM/{noaa}/YYYYDDD_amsu_{noaa}.dat
;
; OUTPUTS:    
;              Monthly gridded means are written to the files:
;
;                 $OutPath/YYYYMM/YYYYDDD_{noaa}_nc
;
; AUTHOR:      
;              Liang Peng / Ben Ho (?)
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
; 
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;                noaa = The name of the NOAA satellite to process.
;                           values = {'noaa15', 'noaa16', 'noaa18', 
;                                     'noaa19', 'noaam2'}
;                chan = AMSU channel to process.
;                           values = {7}
;   YEAR_BEG,JDAY_BEG = The begining year and Julian day to process
;   YEAR_END,JDAY_END = The ending year and Julian day to process
;   InPath            = The root path to the NOAA AMSU Brightness Temps files.
;   OutPath           = The root path for the gridded ASCII files to be generated.
;
;                       Path values should contain a trailing '/'
;
;**************************************************************************
     noaa='noaa15'
     chan=7
   JD_BEG=julday(1,1,2001)
   JD_END=julday(1,1,2001)
   InPath='../Data/tmp/linfit/extract/'
  OutPath='../Data/tmp/linfit/bin/'
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Loop over the years to process
;-----------------------------------
for JD_loop=JD_BEG,JD_END do begin


  ; Set time virables
  ;-----------------------------------------
  caldat,JD_loop,month_loop,dom_loop,year_loop

  ii_wetrt=0L

  ; Construct the input file name and open it
  ;--------------------------------------------
  infile_noaa=InPath+noaa+'/'+string( year_loop,format='(i4.4)')     $
	   +'/amsu_'+noaa+'_'+string( year_loop,format='(i4.4)')+'.' $
			     +string(  day_loop,format='(i3.3)')+'.dat'
  print,'infile=',infile_noaa
  openr,6,infile_noaa,ERROR=err

  ; if there was an error, goto next day
  ;--------------------------------------
  if(err ne 0) then begin
    print,' Input open error: err=',err
    goto,NEXT_DAY
  endif

  ; Get the number of records in the file.
  ; if there are none, then goto next day
  ;----------------------------------------
  inum2=long(file_lines(infile_noaa))
  print,"NOAA inum2=",inum2
  if(inum2 le 2) then begin
    print,'there is no noaa data in',day_loop
    goto,NEXT_DAY
  endif

  ; read in NOAA AMSU data and close the file.
  ;-------------------------------------------
  temp = fltarr(22,inum2)
  readf,6,temp
  close,6

  ; restrict indicies to only include valid temperatures
  ;-------------------------------------------------------
  valid_id=where((temp[chan+6,*] gt 0 and temp[chan+6,*] lt 300),valid_num)
  print,"NOAA inum2=",inum2, '  valid num = ',valid_num
  if(valid_num eq 0) then begin
    print,'no valid noaa data today',infile_noaa
    goto,NEXT_DAY
  endif

  ; extract the desired values
  ;-----------------------------
  day_noaa =temp[1,valid_id]
  time_noaa=temp[2,valid_id]
  lat_noaa =temp[3,valid_id]
  lon_noaa =temp[4,valid_id]  
  sza_noaa =temp[5,valid_id]
  sat_noaa =temp[6,valid_id]
  bt_noaa  =temp[chan+6,valid_id]

  ; Construct output file name and open it.
  ;------------------------------------------
  outfile=OutPath+noaa+'/bin_'+noaa+'_'               $
		 +string( year_loop,format='(i4)'  )  $
		 +string(month_loop,format='(i2.2)')  $
		 +'_chan'+strcompress(string(chan),/remove_all)+'_nc'


  ; Generate gridded data
  ;------------------------------------------
  latbin = 2.5
  lonbin = 2.5
  ilat = fix((lat+90.0)/latbin)<(180./latbin-1)
  ilon = fix((lon+180.0)/lonbin)<(360./lonbin-1)
  ni =  ilon+ilat*144
  sni = sort(ni)
  ni = ni[sni]
  ui = uniq(ni)
  ni_ui = ni[ui]
  num = fltarr(144,72)
  if n_elements(ui) eq 1 then begin
    num[ni_ui] = n_elements(ni)
  endif else begin
    tnum = [ui[0]+1,ui[1:n_elements(ui)-1]-ui[0:n_elements(ui)-2]]
    num[ni_ui] = tnum
  endelse
  sid = string(id,format='(i2.2)')
  vl = double(bt_noaa[sni])
  vl_csum  = total(vl,/CUMULATIVE)
  vl_csum_ui = vl_csum[ui]
  if n_elements(ui) eq 1 then begin
    vl_mean = total(vl)/n_elements(vl)
  endif else begin
    vl_sum = [vl_csum_ui[0],vl_csum_ui[1:n_elements(ui)-1]-vl_csum_ui[0:n_elements(ui)-2]]
    vl_mean = vl_sum/tnum
  endelse
  mval = dblarr(144,72)-999
  mval[ni_ui] = vl_mean
  ptr_var = ptrarr(2,/allocate_heap)
  *ptr_var[0] = mval
  *ptr_var[1] = num

  ;
  ; Write data to output file
  ;-------------------------------------
  var = ['chn_bin','num']
  outdir = file_dirname(outfile)
  if ~file_test(outdir) then spawn,'mkdir -p '+outdir
  fid = NCDF_CREATE(outfile,/CLOBBER)
  lat_dim = NCDF_DIMDEF(fid, 'lat', 72)
  lon_dim = NCDF_DIMDEF(fid,'lon', 144)
  NCDF_CONTROL, fid, /FILL
  vid = lonarr(n_elements(var))
  for iv = 0,n_elements(var)-1 do begin
    type = size(*ptr_var[iv],/tname)
    rt=execute('vid[iv] = NCDF_VARDEF(fid, var[iv], [lon_dim,lat_dim],/'+type+')');
  endfor
  NCDF_CONTROL, fid, /ENDEF
  for iv = 0,n_elements(var)-1 do begin
    NCDF_VARPUT, fid, vid[iv], *ptr_var[iv]
  endfor
  NCDF_CLOSE, fid ; Close the netCDF file.

  ; goto target for next data
  ;----------------------------
  NEXT_DAY:
endfor  ; JD_loop=JD_BEG,JD_END 
;
; End Program
;----------------
end
