@readstd.pro
pro extract_raob_profiles
;=====================================================================
;@***h*  MW_Sounder_FCDR/extract_raob_profiles.pro
;
; NAME
;        extract_raob_profiles.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              This program loops over the specified range of julian days 
;              and reads in all of the RAOB profiles for each day.
;
;              The daily profiles from RAOB files are interpolated to the 
;              100 fixed pressure levels used by the AMSU forward model. 
;              Missing data values are replaced using Standard atmosphere 
;              values.
;
;              The resulting interpolated profiles are related values for 
;              each Julian day are stored for later use.
;
; EXTERNALS:   
;              readstd.pro
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
 
; INPUTS:      
;              RAOB profile files should be stored with the following 
;              directory structure and file format:
;
;                $InPath/raob_YYYY.DDD.txt
;
; OUTPUTS:     
;              Output ASCII files are stored with the following directory 
;              structure and file format:
;
;                $OutPath/YYYY/Prf_YYYY.DDD.dat'
;
; AUTHOR:      
;              Xinjia Zhou / Ben Ho (?)
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;             StdPath = Path to the Standard Atmosphere files for Temperature 
;                       and Water Vapor {'stdmoist.gr', 'stdtemp.gr'}
;   YEAR_BEG,JDAY_BEG = The begining year and Julian day to process
;   YEAR_END,JDAY_END = The ending year and Julian day to process
;   InPath            = The root path to the RAOB data files.
;   OutPath           = The root path for the ASCII files to be generated.
;
;                       Path values should contain a trailing '/'
;**************************************************************************
 StdPath='./Anc_Data/'
YEAR_BEG=2006
JDAY_BEG=300
YEAR_END=2014
JDAY_END=365
  InPath='../Data/Input/raob/'
 OutPath='../Data/ASCII_tmp/extract/raob/'
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Set the fixed vertical levels defined in the AMSU forward model
;--- --------------------------------------------------------------------
Ps = [   0.0050,    0.0161,    0.0384,    0.0769,    0.1370,            $
         0.2244,    0.3454,    0.5064,    0.7140,    0.9753,    1.2972, $
         1.6872,    2.1526,    2.7009,    3.3398,    4.0770,    4.9204, $
         5.8776,    6.9567,    8.1655,    9.5119,   11.0038,   12.6492, $
        14.4559,   16.4318,   18.5847,   20.9224,   23.4526,   26.1829, $
        29.1210,   32.2744,   35.6505,   39.2566,   43.1001,   47.1882, $
        51.5278,   56.1260,   60.9895,   66.1253,   71.5398,   77.2396, $
        83.2310,   89.5204,   96.1138,  103.0172,  110.2366,  117.7775, $
       125.6456,  133.8462,  142.3848,  151.2664,  160.4959,  170.0784, $
       180.0183,  190.3203,  200.9887,  212.0277,  223.4415,  235.2338, $
       247.4085,  259.9691,  272.9191,  286.2617,  300.0000,  314.1369, $
       328.6753,  343.6176,  358.9665,  374.7241,  390.8926,  407.4738, $
       424.4698,  441.8819,  459.7118,  477.9607,  496.6298,  515.7200, $
       535.2322,  555.1669,  575.5248,  596.3062,  617.5112,  639.1398, $
       661.1920,  683.6673,  706.5654,  729.8857,  753.6275,  777.7897, $
       802.3714,  827.3713,  852.7880,  878.6201,  904.8659,  931.5236, $
       958.5911,  986.0666,  990.9476,  995.2319, 1008.1700]
nlev = n_elements(ps)

; Set seasonal flags (monthly)
;-----------------------------
moni = [1, 1, 1, -1, -1, -1, -1, -1, -1, 1, 1, 1]

; Set MAXLEN larger than the max number of profiles in one day
;----------------------------------------------------------
MAXLEN = 5000L

;
; Search for standard atmospheric information files
; read in temperature and moisture values.
;-----------------------------------------------------
fstdt=file_search(StdPath,'stdtemp.gr' ,count=cstdt)
fstdm=file_search(StdPath,'stdmoist.gr',count=cstdm)
if((cstdt eq 0) or (cstdm eq 0)) then begin
  print,' ERROR: Cannot find the standard atmosphere files'
  STOP
endif
readstd, fstdt, Ps, T_climate
readstd, fstdm, Ps, vp_climate

;
; get starting and ending Julian days
;--------------------------------------------
JulianDay_BEG = julday(1,0,YEAR_BEG)+JDAY_BEG
JulianDay_END = julday(1,0,YEAR_END)+JDAY_END

;   
; Loop over the Julian days to process
;----------------------------------
for JulianDay = JulianDay_BEG,JulianDay_END do begin

  ; get month and day of month
  ;--------------------------------------------
  caldat,JulianDay,month,dom,year_loop
  day_loop = JulianDay - julday(1,0,year_loop)

  ; Get raob files for current julday day
  ;--------------------------------------------
  infile=Inpath+string(year_loop,format='(i4)')+'/raob_'+$
                string(year_loop,format='(i4)')+$
            '.'+string(day_loop, format='(i3.3)')+'.dat'

  ; verify that the RAOB profile exists
  ;-------------------------------------
  if ~file_test(infile) then begin
    print,'RAOB file do not exist',infile
    goto,NEXT_PROFILE
  endif

  ; Allocate Space
  ;--------------------
  Ts   = fltarr(maxlen, nlev)
  svps = fltarr(maxlen, nlev)-9.
  vps  = fltarr(maxlen, nlev)-9.
  yrz = intarr(maxlen)
  dayz = intarr(maxlen)
  monz = intarr(maxlen)
  timez = fltarr(maxlen)
  laty = fltarr(maxlen)
  lonx = fltarr(maxlen)
  ratpz = fltarr(maxlen)
  cornz = fltarr(maxlen)
  sircz = fltarr(maxlen)
  ik_total=intarr(maxlen)
  Pcosmi    = fltarr(maxlen,500)
  Vpcosmi   = fltarr(maxlen,500)
  svpcosmi  = fltarr(maxlen,500)
  Tcosmi    = fltarr(maxlen,500)

  ;read ds351 data
  ;------------------------------------------
  openr,lun1,infile,/get_lun
  in=0L
  while( ~eof(lun1) ) do begin
    readf,lun1,yr,wholeday,hr,mint,lat,lon,ratp,corn,sirc
    yrz[in]=yr
    caldat,julday(1,wholeday,yr),monzi,dayzi
    monz[in]=monzi
    dayz[in]=wholeday;
    timez[in]=float(hr)+float(mint)/60.
    laty[in]=lat
    lonx[in]=lon
    ratpz[in]=ratp
    cornz[in]=corn
    sircz[in]=sirc

    ik=0L
    while( ~eof(lun1) ) do begin
      point_lun,-1*lun1,pos1
      readf, lun1,P,geo1,geo2,T,Vp,svp,Ref

      if(P eq yr) then begin
	point_lun,lun1,pos1
	break
      endif

      if P eq -999. or T ge 50. then goto,jump1

      Pcosmi(in,ik)   =P
      Tcosmi(in,ik)   =T
      Vpcosmi(in,ik)  =Vp
      svpcosmi(in,ik)  =svp

      ik=ik+1
      jump1:
    endwhile ;end this profile
    if ik eq 0 then goto,jump_bad

    ;
    ;interpolate data into the fix Ps levels
    ;-------------------------------------------
    ;temperature and saturation water vapor
    ;-------------------------------------------
    tmp = where(ps le max(Pcosmi(in,0:ik-1))  and ps ge min(Pcosmi(in,0:ik-1)), c, complement = ntmp, ncomplement = nc)
    if c le 3 then goto,jump_bad ;no enough data in a profile
    Ts[in, tmp]   = interpol(Tcosmi(in,0:ik-1),   Pcosmi(in,0:ik-1), Ps[tmp])+273.
    svps[in, tmp]   = interpol(svpcosmi(in,0:ik-1),   Pcosmi(in,0:ik-1), Ps[tmp])
    if nc ge 1 then begin
      Ts[in, ntmp]    = !Values.F_NaN 
    endif

    ;water vapor pressure, there are some levels without valid observation
    ;-------------------------------------------
    jj=0
    while jj le ik-1 do begin
      ;find a new valid level
      ;------------------------
      if Vpcosmi(in,jj) ge 0 then begin
	kk = jj+1
	while kk le ik-1 do begin
	  if Vpcosmi(in,kk) lt 0 then begin 
            ;find a unvalid point
            ;----------------------
	    if kk-1-jj ge 2 then begin
	       tmp2 = where(ps le Pcosmi(in,jj)  and ps ge Pcosmi(in,kk-1), c2)
	       if c2 ge 1 then begin
		 vps[in, tmp2]   = interpol(vpcosmi(in,jj:kk-1),   Pcosmi(in,jj:kk-1), Ps[tmp2])
	       endif
	       break          
	    endif else begin
	       break
	    endelse        
	  endif else if kk eq ik-1 then begin 
            ;reach the top
            ;---------------------
	    if kk-jj ge 2 then begin
	       tmp2 = where(ps le Pcosmi(in,jj)  and ps ge Pcosmi(in,kk), c2)
	       if c2 ge 1 then begin
		 vps[in, tmp2]   = interpol(vpcosmi(in,jj:kk),   Pcosmi(in,jj:kk), Ps[tmp2])
	       endif
	       break          
	    endif else begin
	       break
	    endelse          
	  endif else begin 
            ;valid level goes on
            ;----------------------
	    kk=kk+1
	  endelse
	endwhile 
	jj=kk+1
      endif else begin
	jj=jj+1
      endelse
    endwhile

    ik_total(in)=ik
    in=in+1

    ;goto next profile
    ;-----------------------
    jump_bad:
  endwhile
  print,'actual inum1=',in

  close,lun1
  free_lun,lun1

  ; Trim array sizes
  ;---------------------------------------------------
  yrz   = yrz[0:in-1]
  monz  = monz[0:in-1]
  dayz  = dayz[0:in-1]
  timez = timez[0:in-1]
  laty  = laty[0:in-1]
  lonx  = lonx[0:in-1]
  ratpz = ratpz[0:in-1]
  cornz = cornz[0:in-1]
  sircz = sircz[0:in-1]
  ik_total=ik_total[0:in-1]
  Pcosmi= Pcosmi[0:in-1,*]
  Tcosmi= Tcosmi[0:in-1,*]
  Ts    = Ts[0:in-1, *]
  vps   = vps[0:in-1, *]
  svps  = svps[0:in-1, *]

  ;sort by lat , from -90 ~ 90
  ;----------------------------------------------------
  sort_lat=sort(laty)
  yrz   =yrz(sort_lat)
  monz  =monz(sort_lat)
  dayz  =dayz(sort_lat)
  timez =timez(sort_lat)
  laty  =laty(sort_lat)
  lonx  =lonx(sort_lat)
  ratpz =ratpz(sort_lat)
  cornz =cornz(sort_lat)
  sircz =sircz(sort_lat)
  ik_total=ik_total[sort_lat]
  Pcosmi= Pcosmi[sort_lat,*]
  Tcosmi= Tcosmi[sort_lat,*]
  Ts    =Ts(sort_lat,*)
  vps   =vps(sort_lat,*)
  svps  =svps(sort_lat,*)

  ; Allocate space for reference values
  ;---------------------------------------
  Ts0    = fltarr(in,100)
  vps0   = fltarr(in,100)
  Valid= intarr(in,100)+1

  dum = where(finite(Ts) eq 0, c)

  if c ne 0 then begin ;there are Nan
    valid(dum)=0
   
    ;preparing the temperature information (from standard atmosphere) at each grid
    ;-----------------------------------------------------------------------------

    ; Load reference (1) tropical (20N ~ 20S)
    ;----------------------------
    tmp_lat = where(abs(laty) le 20, c1)
    for i = 0, c1-1 do begin
      Ts0[tmp_lat[i], *]  = T_climate[0, *]
    endfor

    ; Load reference (2) midlat (20N ~ 60N) summer
    ;----------------------------
    tmp_lat = where(abs(laty+moni[monz-1]*40) le 20, c2)
    for i = 0, c2-1 do begin
      Ts0[tmp_lat[i], *] = T_climate[1, *]
    endfor

    ; Load reference (3) midlat (20N ~ 60N) winter
    ;----------------------------
    tmp_lat = where(abs(laty-moni[monz-1]*40) le 20, c3)
    for i = 0, c3-1 do begin
      Ts0[tmp_lat[i], *] = T_climate[2, *]
    endfor

    ; Load reference (4) subarctic (60N ~ 90N) summer
    ;----------------------------
    tmp_lat = where(abs(laty+moni[monz-1]*75) le 15, c4)
    for i = 0, c4-1 do begin
      Ts0[tmp_lat[i], *] = T_climate[3, *]
    endfor

    ; Load reference (5) subarctic (60N ~ 90N) winter
    ;----------------------------
    tmp_lat = where(abs(laty-moni[monz-1]*75) le 15, c5)
    for i = 0, c5-1 do begin
      Ts0[tmp_lat[i], *] = T_climate[4, *]
    endfor

    ; replace missing Temps with refernce temperature
    ;------------------------------------------------
    Ts[dum]   = Ts0[dum]
  endif

  ; open output file
  ;-------------------------------------
  outfile = OutPath+string(year_loop,format='(i4)')+'/'+ $
             'Prf_'+string(year_loop,format='(i4)')+ $
                    string(day_loop,format='(i3.3)')+'.dat'
  print,outfile
  err=file_test(OutPath,/DIRECTORY,/WRITE)
  if(err ne 1) then begin
   print,' OutPath does not exist or does not have write permission'
   print,' OutPath=',OutPath
   STOP
  endif
  outdir  = file_dirname(outfile)
  err=file_test(outdir,/DIRECTORY,/WRITE)
  if(err ne 1 ) then begin
   file_mkdir,outdir
  endif
  openw,5,outfile,ERROR=err
  if(err ne 0) then begin
   print,' ERROR opening output file: err=',err
   print,' outfile=',outfile
   STOP
  endif

  ; save results in output file
  ;-------------------------------------
  for ii=0,in-1 do begin
    printf,5,yrz[ii],dayz[ii],timez[ii],ratpz[ii],cornz[ii],$
             sircz[ii],laty[ii],lonx[ii],ik_total(ii),$
    format='(1x,i4,1x,i3,1x,f7.3,2(2x,i3),2x,i4,1x,2(f7.2,2x),i3)'
    for k = 0, ik_total(ii)-1 do printf, 5,k,Pcosmi[ii,k], Tcosmi[ii,k],$
                                 format='(i4,2x, 3(f10.4, 2x))'
    for k = 0, nlev-1 do printf,5,k,Ps[k], Ts[ii,k],vps[ii,k],svps[ii,k], valid[ii,k],$
                         format='(i4,2x, 2(f10.4, 2x),2(f15.8),2x,i)'
  endfor
  close,5
  free_lun,5

  ; goto target for next day
  ;---------------------------
  NEXT_PROFILE:
endfor ; JulianDay = JulianDay_BEG,JulianDay_END
; 
; End Program
;---------------
end
