pro gen_product
;=====================================================================
;@***h*  MW_Sounder_FCDR/gen_product.pro
;
; NAME
;        gen_product.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION:
;              Read in monthly brightness temperature to generate the 
;              products of climatology and anomaly and saved in 
;              netCDF format.
;
; EXTERNALS:
;              none.
;
; USAGE:
;              The RUN parameters specifying the data to process are set
;              directly below this header. The user needs to edit these
;              values to process the desired values and then execute the
;              program from within IDL.
;
; INPUTS:
;              Monthly brightness temperatures are read from file with
;              the following directory structure and file format:
;
;                $InPath/YYYY_MM_nc
;
; OUTPUTS:
;              Monthly brightness temperatures are written to netCDF files:
;
;                $OutPath/RO-CAL-BT-CDR_V02R00_MON_s*.nc
;
;              Brightness temperatures climatology are written to netCDF files:
;
;                $OutPath/RO-CAL-BT-CDR_V02R00_CLIM_s*.nc
;
;              Brightness temperatures anomaly are written to netCDF files:
;
;                $OutPath/RO-CAL-BT-CDR_V02R00_ANOM_s*.nc
;
; AUTHOR:
;              Liang Peng
;
; REVISIONS:
;              Liang Peng (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
; year_start = The beginning year to process
;   year_end = The ending year to process
;month_start = The beginning month to process
;  month_end = The ending month to process
;yearc_start = The beginning year for Climatology Calculation
;  yearc_end = The ending year for Climatology Calculation
;     InPath = The root path to the monthly brightness temperatures
;    OutPath = The root path for the monthly brightness temperatures,
;              climatology and anomaly
;                       Path values should contain a trailing '/'
;
;**************************************************************************
 year_start = 1986
   year_end = 2014
month_start = 11
  month_end = 12
yearc_start = 1987
  yearc_end = 2014
     InPath = '../Data/tmp/monthly/'
    OutPath = '../Data/Output/'
;*********************************************************
; End of RUN parameters
;*********************************************************

; Definite Longitude and Latitude values
;-----------------------------------
lon_grid = 2.5
lat_grid = 2.5
nlon = 360./lon_grid
nlat = 180./lat_grid
lat = findgen(nlat)*lat_grid- 90+lat_grid/2.
lon = findgen(nlon)*lon_grid-180+lon_grid/2.
lat_bounds = fltarr(2,nlat)
lon_bounds = fltarr(2,nlon)
lat_bounds[0,*] = lat - lat_grid/2
lat_bounds[1,*] = lat + lat_grid/2
lon_bounds[0,*] = lon - lon_grid/2
lon_bounds[1,*] = lon + lon_grid/2

; Generate the juldays for the months to deal with
;------------------------------------
jdms = timegen(start=julday(month_start,15,year_start),final=julday(month_end,15,yearc_end),unit='month')
njdm = n_elements(jdms)

; Initial BT data
;-----------------------------------
BT_monthly = fltarr(nlon,nlat,njdm)-999
BT_anomaly = fltarr(nlon,nlat,njdm)-999
BT_climatology = fltarr(nlon,nlat,12)-999

; Loop over monthes to process
;-----------------------------------
for ijdm = 0,n_elements(jdms)-1 do begin

  ; Generate the month,and year from Julian day
  ;-----------------------------
  caldat,jdms[ijdm],mmonth,mdom,myear

  ; Read in BTs in each month
  ;-----------------------------
  Infile = InPath+string(myear,f='(i4.4)')+string(mmonth,f='(i2.2)')+'.sav'
  if file_test(Infile) then begin 
    restore,infile

    ; Copy values for later use
    ;------------------------------
    BT_monthly[*,*,ijdm] = BT_month
  endif

endfor ; loop of month

; Generate BT Climatology and Anomaly
;--------------------------------
caldat,jdms,month,dom,year
for im = 1,12 do begin
  id = where(month eq im and year ge yearc_start and year le yearc_end,nid)
  if nid gt 0 then begin
    if nid eq 1 then begin
      BT_climatology[*,*,im-1] = BT_monthly[*,*,id]
    endif else begin
      BT_climatology[*,*,im-1] = total(BT_monthly[*,*,id] > 0,3)/total(BT_monthly[*,*,id] gt 0,3)
    endelse
    idanom = where(month eq im)
    BT_clim = BT_climatology[*,*,im-1]
    id_fin = where(BT_clim > 0,complement=idnan)
    BT_clim[idnan] = !values.d_nan
    for kid = 0,n_elements(idanom)-1 do begin
      BT_anomaly[*,*,idanom[kid]] = BT_monthly[*,*,idanom[kid]] - BT_clim
    endfor 
  endif
endfor
id_nan = where(~finite(BT_climatology))
if id_nan[0] gt -1 then begin
  BT_climatology[id_nan] = -999
endif
id_fin = where(finite(BT_anomaly) and BT_monthly gt 0,complement=id_nan)
if id_nan[0] gt -1 then begin
  BT_anomaly[id_nan] = -999
endif

;
; Output Monthly Brightness temperatures
;--------------------------------
; Time variables
;------------------
time_bounds = fltarr(2,njdm)
time_bounds[0,*] = julday(month,1,year)-julday(1,1,1970)
time_bounds[1,*] = julday(month+1,0,year)-julday(1,1,1970)
time = total(time_bounds,1)/2.
ym_start = string(year[0],f='(i4.4)')+string(month[0],f='(i2.2)')
ym_end = string(year[njdm-1],f='(i4.4)')+string(month[njdm-1],f='(i2.2)')
ctime = systime(/julian)
ctime = julday(12,4,2015)
caldat,ctime,cmonth,cdom,cyear,chour,cmin,csec
ymd_create = string(cyear,f='(i4.4)')+string(cmonth,f='(i2.2)')+string(cdom,f='(i2.2)')

; File name and dimensions
;------------------
Outfile = 'MLT-TTS-UCAR_V02R00_MON_s'+ym_start+'_e'+ym_end+'_c'+ymd_create+'.nc'
fid = NCDF_CREATE(Outpath+Outfile,/CLOBBER)
dim = size(BT_monthly,/dimensions)
latitude_dim = NCDF_DIMDEF(fid, 'lat', dim[1])
longitude_dim = NCDF_DIMDEF(fid, 'lon', dim[0])
time_dim = NCDF_DIMDEF(fid, 'time', dim[2])
bnds_dim = NCDF_DIMDEF(fid, 'bnds', 2)

; Variables definition
;------------------
NCDF_CONTROL, fid, /FILL
vid_time = NCDF_VARDEF(fid, 'time', time_dim,/float)
NCDF_ATTPUT, fid, vid_time, 'units', 'days since 1970-01-01 00:00:00'
NCDF_ATTPUT, fid, vid_time, 'long_name', 'time'
NCDF_ATTPUT, fid, vid_time, 'standard_name', 'time'
NCDF_ATTPUT, fid, vid_time, 'axis', 'T'
NCDF_ATTPUT, fid, vid_time, 'bounds', 'time_bounds'
NCDF_ATTPUT, fid, vid_time, 'missing_value', -999.
vid_lon = NCDF_VARDEF(fid, 'lon', longitude_dim,/float)
NCDF_ATTPUT, fid, vid_lon, 'units', 'degrees_east'
NCDF_ATTPUT, fid, vid_lon, 'long_name', 'longitude'
NCDF_ATTPUT, fid, vid_lon, 'standard_name', 'longitude'
NCDF_ATTPUT, fid, vid_lon, 'axis', 'X'
NCDF_ATTPUT, fid, vid_lon, 'bounds', 'lon_bounds'
NCDF_ATTPUT, fid, vid_lon, 'missing_value', -999.
vid_lat = NCDF_VARDEF(fid, 'lat', latitude_dim,/float)
NCDF_ATTPUT, fid, vid_lat, 'units', 'degrees_north'
NCDF_ATTPUT, fid, vid_lat, 'long_name', 'latitude'
NCDF_ATTPUT, fid, vid_lat, 'standard_name', 'latitude'
NCDF_ATTPUT, fid, vid_lat, 'axis', 'Y'
NCDF_ATTPUT, fid, vid_lat, 'bounds', 'lat_bounds'
NCDF_ATTPUT, fid, vid_lat, 'missing_value', -999.
vid_time_bounds = NCDF_VARDEF(fid, 'time_bounds', [bnds_dim,time_dim],/float)
NCDF_ATTPUT, fid, vid_time_bounds, 'valid_min', 0.
NCDF_ATTPUT, fid, vid_time_bounds, 'valid_max', julday(12,31,year[njdm-1])-julday(1,1,1970)
NCDF_ATTPUT, fid, vid_time_bounds, 'coordinates', 'time'
vid_lon_bounds = NCDF_VARDEF(fid, 'lon_bounds', [bnds_dim,longitude_dim],/float)
NCDF_ATTPUT, fid, vid_lon_bounds, 'valid_min', -180.
NCDF_ATTPUT, fid, vid_lon_bounds, 'valid_max', 180.
NCDF_ATTPUT, fid, vid_lon_bounds, 'coordinates', 'lon'
vid_lat_bounds = NCDF_VARDEF(fid, 'lat_bounds', [bnds_dim,latitude_dim],/float)
NCDF_ATTPUT, fid, vid_lat_bounds, 'valid_min', -90.
NCDF_ATTPUT, fid, vid_lat_bounds, 'valid_max', 90.
NCDF_ATTPUT, fid, vid_lat_bounds, 'coordinates', 'lat'
vid_BT = NCDF_VARDEF(fid, 'brightness_temperature', [longitude_dim,latitude_dim,time_dim],/float)
NCDF_ATTPUT, fid, vid_BT, 'units', 'K'
NCDF_ATTPUT, fid, vid_BT, 'long_name', 'NOAA Climate Data Record of Mean Layer Temperature - UCAR (Upper Trop & Lower Strat) Monthly Mean'
NCDF_ATTPUT, fid, vid_BT, 'standard_name', 'brightness_temperature'
NCDF_ATTPUT, fid, vid_BT, 'valid_min', 100.
NCDF_ATTPUT, fid, vid_BT, 'valid_max', 300.
NCDF_ATTPUT, fid, vid_BT, 'missing_value', -999.
NCDF_ATTPUT, fid, vid_BT, 'coordinates', 'time lat lon'

; Global Attributies
;--------------------
NCDF_ATTPUT, fid, 'Conventions', 'CF-1.6', /global
NCDF_ATTPUT, fid, 'title', 'NOAA Climate Data Record of Mean Layer Temperature - UCAR (Upper Trop & Lower Strat) Monthly Mean', /global
NCDF_ATTPUT, fid, 'product_version', 'v02r00', /global
NCDF_ATTPUT, fid, 'software_version_id', 'gov.noaa.ncdc:/svn-repos/rsad/CDR/RO_CAL_BT/tags/v02r00/', /global
NCDF_ATTPUT, fid, 'source', 'UCAR COSMIC Data Analysis and Archival Center version 2010.2640', /global
NCDF_ATTPUT, fid, 'Metadata_Conventions', 'CF-1.6, Unidata Dataset Discovery v1.0, NOAA CDR v2.0, GDS v2.0', /global
NCDF_ATTPUT, fid, 'standard_name_vocabulary', 'CF Standard Name Table (v29, 08 July 2015)', /global
NCDF_ATTPUT, fid, 'id', outfile, /global
NCDF_ATTPUT, fid, 'naming_authority', 'gov.noaa.ncdc', /global
NCDF_ATTPUT, fid, 'date_created', string(cyear,f='(i4.4)')+'-'+string(cmonth,f='(i2.2)')+'-'+string(cdom,f='(i2.2)')+'T'+$
                   string(chour,f='(i4.4)')+':'+string(cmin,f='(i2.2)')+':'+string(csec,f='(i2.2)')+'Z', /global
NCDF_ATTPUT, fid, 'license', 'No constraints on data access or use', /global
NCDF_ATTPUT, fid, 'summary', 'The product is the monthly mean of adjusted MSU channel 3 and AMSU channel 7 brightness temperatures. Calculated brightness temperatures from an AMSU forward model using temperature and moisture profiles from radiosonde observations which identified by GPSRO were used to calibrate the channel 7 measurements from polar orbiters to a common reference.', /global
NCDF_ATTPUT, fid, 'keywords', 'Earth Science > Spectral/Engineering > Microwave > Brightness Temperature', /global
NCDF_ATTPUT, fid, 'keywords_vocabulary', 'NASA Global Change Master Directory (GCMD) Earth Science Keywords, Version 8.1', /global
NCDF_ATTPUT, fid, 'cdm_data_type', 'Grid', /global
NCDF_ATTPUT, fid, 'institution', 'UCAR/COSMIC', /global
NCDF_ATTPUT, fid, 'geospatial_lat_min', min(lat), /global
NCDF_ATTPUT, fid, 'geospatial_lat_max', max(lat), /global
NCDF_ATTPUT, fid, 'geospatial_lon_min', min(lon), /global
NCDF_ATTPUT, fid, 'geospatial_lon_max', max(lon), /global
NCDF_ATTPUT, fid, 'geospatial_lat_units', 'degrees_north', /global
NCDF_ATTPUT, fid, 'geospatial_lon_units', 'degrees_east', /global
NCDF_ATTPUT, fid, 'geospatial_lat_resolution', lat_grid, /global
NCDF_ATTPUT, fid, 'geospatial_lon_resolution', lon_grid, /global
NCDF_ATTPUT, fid, 'time_coverage_start', string(year_start,f='(i4.4)')+'-'+string(month_start,f='(i2.2)')+'-01T00:00:00Z', /global
NCDF_ATTPUT, fid, 'time_coverage_end', string(year_end,f='(i4.4)')+'-'+string(month_end,f='(i2.2)')+'-'+$
                   string(julday(month_end+1,0,year_end)-julday(month_end,0,year_end),f='(i2.2)')+'T23:59:59Z', /global
NCDF_ATTPUT, fid, 'time_coverage_duration', 'P1Y', /global
NCDF_ATTPUT, fid, 'time_coverage_resolution', 'P1M', /global
NCDF_ATTPUT, fid, 'cdr_program', 'NOAA Climate Data Record Program for satellites', /global
NCDF_ATTPUT, fid, 'cdr_variable', 'brightness temperature', /global
NCDF_ATTPUT, fid, 'Metadata_Link', 'gov.noaa.ncdc:C00944', /global
NCDF_ATTPUT, fid, 'platform', 'NOAA-9 > National Oceanic & Atmospheric Administration-9; NOAA-10 > National Oceanic & Atmospheric Administration-10; NOAA-11 > National Oceanic & Atmospheric Administration-11; NOAA-12 > National Oceanic & Atmospheric Administration-12; NOAA-14 > National Oceanic & Atmospheric Administration-14; NOAA-15 > National Oceanic & Atmospheric Administration-15; NOAA-16 > National Oceanic & Atmospheric Administration-16; NOAA-18 > National Oceanic & Atmospheric Administration-18; NOAA-19 > National Oceanic & Atmospheric Administration-19; CHAMP > Challenging Minisatellite Payload, Gravity Recovery and Climate Experiment', /global
NCDF_ATTPUT, fid, 'sensor', 'GPS > Global Positioning System; MSU > Microwave Sounding Unit; AMSU > Advanced Microwave Sounding Unit', /global
NCDF_ATTPUT, fid, 'spatial_resolution', '2.5 x 2.5 degree', /global
NCDF_ATTPUT, fid, 'processing_level', 'NOAA Level 3', /global
NCDF_CONTROL, fid, /ENDEF

; Write values to the outfile
;-----------------------
NCDF_VARPUT, fid, vid_time, time
NCDF_VARPUT, fid, vid_lat, lat
NCDF_VARPUT, fid, vid_lon, lon
NCDF_VARPUT, fid, vid_time_bounds, time_bounds
NCDF_VARPUT, fid, vid_lat_bounds, lat_bounds
NCDF_VARPUT, fid, vid_lon_bounds, lon_bounds
NCDF_VARPUT, fid, vid_BT, BT_monthly
NCDF_CLOSE, fid


;
; Output Brightness Temperature Climatology
;--------------------------------
; Time values
;------------------
ym_start = string(yearc_start,f='(i4.4)')+string(1,f='(i2.2)')
ym_end = string(yearc_end,f='(i4.4)')+string(12,f='(i2.2)')

; File name and dimensions
;------------------
Outfile = 'MLT-TTS-UCAR_V02R00_CLIM_s'+ym_start+'_e'+ym_end+'_c'+ymd_create+'.nc'
fid = NCDF_CREATE(Outpath+Outfile,/CLOBBER)
dim = size(BT_climatology,/dimensions)
latitude_dim = NCDF_DIMDEF(fid, 'lat', dim[1])
longitude_dim = NCDF_DIMDEF(fid, 'lon', dim[0])
time_dim = NCDF_DIMDEF(fid, 'time', dim[2])
bnds_dim = NCDF_DIMDEF(fid, 'bnds', 2)
climatology_bounds = fltarr(2,12)
climatology_bounds[0,*] = findgen(12)+1
climatology_bounds[1,*] = findgen(12)+1
time = fltarr(12)
for imonth = 1,12 do begin
  time[imonth-1] = (julday(imonth,1,1970)+julday(imonth+1,0,1970))/2.-julday(1,1,1970)
  climatology_bounds[0,imonth-1] = julday(imonth,1,yearc_start)-julday(1,1,1970)
  climatology_bounds[1,imonth-1] = julday(imonth+1,0,yearc_end)-julday(1,1,1970)
endfor

; Variables definition
;------------------
NCDF_CONTROL, fid, /FILL
vid_time = NCDF_VARDEF(fid, 'time', time_dim,/float)
NCDF_ATTPUT, fid, vid_time, 'units', 'days since 1970-01-01 00:00:00'
NCDF_ATTPUT, fid, vid_time, 'long_name', 'time'
NCDF_ATTPUT, fid, vid_time, 'standard_name', 'time'
NCDF_ATTPUT, fid, vid_time, 'axis', 'T'
NCDF_ATTPUT, fid, vid_time, 'climatology', 'climatology_bounds'
NCDF_ATTPUT, fid, vid_time, 'missing_value', -999.
vid_lon = NCDF_VARDEF(fid, 'lon', longitude_dim,/float)
NCDF_ATTPUT, fid, vid_lon, 'units', 'degrees_east'
NCDF_ATTPUT, fid, vid_lon, 'long_name', 'longitude'
NCDF_ATTPUT, fid, vid_lon, 'standard_name', 'longitude'
NCDF_ATTPUT, fid, vid_lon, 'axis', 'X'
NCDF_ATTPUT, fid, vid_lon, 'bounds', 'lon_bounds'
NCDF_ATTPUT, fid, vid_lon, 'missing_value', -999.
vid_lat = NCDF_VARDEF(fid, 'lat', latitude_dim,/float)
NCDF_ATTPUT, fid, vid_lat, 'units', 'degrees_north'
NCDF_ATTPUT, fid, vid_lat, 'long_name', 'latitude'
NCDF_ATTPUT, fid, vid_lat, 'standard_name', 'latitude'
NCDF_ATTPUT, fid, vid_lat, 'axis', 'Y'
NCDF_ATTPUT, fid, vid_lat, 'bounds', 'lat_bounds'
NCDF_ATTPUT, fid, vid_lat, 'missing_value', -999.
vid_climatology_bounds = NCDF_VARDEF(fid, 'climatology_bounds', [bnds_dim,time_dim],/float)
NCDF_ATTPUT, fid, vid_climatology_bounds, 'valid_min', julday(1,1,yearc_start)-julday(1,1,1970)
NCDF_ATTPUT, fid, vid_climatology_bounds, 'valid_max', julday(12,31,yearc_end)-julday(1,1,1970)
NCDF_ATTPUT, fid, vid_climatology_bounds, 'coordinates', 'time'
vid_lon_bounds = NCDF_VARDEF(fid, 'lon_bounds', [bnds_dim,longitude_dim],/float)
NCDF_ATTPUT, fid, vid_lon_bounds, 'valid_min', -180.
NCDF_ATTPUT, fid, vid_lon_bounds, 'valid_max', 180.
NCDF_ATTPUT, fid, vid_lon_bounds, 'coordinates', 'lon'
vid_lat_bounds = NCDF_VARDEF(fid, 'lat_bounds', [bnds_dim,latitude_dim],/float)
NCDF_ATTPUT, fid, vid_lat_bounds, 'valid_min', -90.
NCDF_ATTPUT, fid, vid_lat_bounds, 'valid_max', 90.
NCDF_ATTPUT, fid, vid_lat_bounds, 'coordinates', 'lat'
vid_BT = NCDF_VARDEF(fid, 'bt_climatology', [longitude_dim,latitude_dim,time_dim],/float)
NCDF_ATTPUT, fid, vid_BT, 'units', 'K'
NCDF_ATTPUT, fid, vid_BT, 'long_name', 'NOAA Climate Data Record of Mean Layer Temperature - UCAR (Upper Trop & Lower Strat) Climatology'
NCDF_ATTPUT, fid, vid_BT, 'standard_name', 'brightness_temperature'
NCDF_ATTPUT, fid, vid_BT, 'valid_min', 100.
NCDF_ATTPUT, fid, vid_BT, 'valid_max', 300.
NCDF_ATTPUT, fid, vid_BT, 'missing_value', -999.
NCDF_ATTPUT, fid, vid_BT, 'coordinates', 'time lat lon'
NCDF_ATTPUT, fid, vid_BT, 'cell_methods', 'time: mean over years'

; Global Attributies
;--------------------
NCDF_ATTPUT, fid, 'Conventions', 'CF-1.6', /global
NCDF_ATTPUT, fid, 'title', 'NOAA Climate Data Record of Mean Layer Temperature - UCAR (Upper Trop & Lower Strat) Climatology', /global
NCDF_ATTPUT, fid, 'product_version', 'v02r00', /global
NCDF_ATTPUT, fid, 'software_version_id', 'gov.noaa.ncdc:/svn-repos/rsad/CDR/RO_CAL_BT/tags/v02r00/', /global
NCDF_ATTPUT, fid, 'source', 'UCAR COSMIC Data Analysis and Archival Center version 2010.2640', /global
NCDF_ATTPUT, fid, 'Metadata_Conventions', 'CF-1.6, Unidata Dataset Discovery v1.0, NOAA CDR v2.0, GDS v2.0', /global
NCDF_ATTPUT, fid, 'standard_name_vocabulary', 'CF Standard Name Table (v29, 08 July 2015)', /global
NCDF_ATTPUT, fid, 'id', outfile, /global
NCDF_ATTPUT, fid, 'naming_authority', 'gov.noaa.ncdc', /global
NCDF_ATTPUT, fid, 'date_created', string(cyear,f='(i4.4)')+'-'+string(cmonth,f='(i2.2)')+'-'+string(cdom,f='(i2.2)')+'T'+$
                   string(chour,f='(i4.4)')+':'+string(cmin,f='(i2.2)')+':'+string(csec,f='(i2.2)')+'Z', /global
NCDF_ATTPUT, fid, 'license', 'No constraints on data access or use', /global
NCDF_ATTPUT, fid, 'summary', 'The product is the climatology of adjusted MSU channel 3 and AMSU channel 7 brightness temperatures. Calculated brightness temperatures from an AMSU forward model using temperature and moisture profiles from radiosonde observations which identified by GPSRO were used to calibrate the channel 7 measurements from polar orbiters to a common reference.', /global
NCDF_ATTPUT, fid, 'keywords', 'Earth Science > Spectral/Engineering > Microwave > Brightness Temperature', /global
NCDF_ATTPUT, fid, 'keywords_vocabulary', 'NASA Global Change Master Directory (GCMD) Earth Science Keywords, Version 8.1', /global
NCDF_ATTPUT, fid, 'cdm_data_type', 'Grid', /global
NCDF_ATTPUT, fid, 'institution', 'UCAR/COSMIC', /global
NCDF_ATTPUT, fid, 'geospatial_lat_min', min(lat), /global
NCDF_ATTPUT, fid, 'geospatial_lat_max', max(lat), /global
NCDF_ATTPUT, fid, 'geospatial_lon_min', min(lon), /global
NCDF_ATTPUT, fid, 'geospatial_lon_max', max(lon), /global
NCDF_ATTPUT, fid, 'geospatial_lat_units', 'degrees_north', /global
NCDF_ATTPUT, fid, 'geospatial_lon_units', 'degrees_east', /global
NCDF_ATTPUT, fid, 'geospatial_lat_resolution', lat_grid, /global
NCDF_ATTPUT, fid, 'geospatial_lon_resolution', lon_grid, /global
NCDF_ATTPUT, fid, 'time_coverage_start', string(year_start,f='(i4.4)')+'-'+string(month_start,f='(i2.2)')+'-01T00:00:00Z', /global
NCDF_ATTPUT, fid, 'time_coverage_end', string(year_end,f='(i4.4)')+'-'+string(month_end,f='(i2.2)')+'-'+$
                   string(julday(month_end+1,0,year_end)-julday(month_end,0,year_end),f='(i2.2)')+'T23:59:59Z', /global
NCDF_ATTPUT, fid, 'time_coverage_duration', 'P1Y', /global
NCDF_ATTPUT, fid, 'time_coverage_resolution', 'P1M', /global
NCDF_ATTPUT, fid, 'cdr_program', 'NOAA Climate Data Record Program for satellites', /global
NCDF_ATTPUT, fid, 'cdr_variable', 'brightness temperature', /global
NCDF_ATTPUT, fid, 'Metadata_Link', 'gov.noaa.ncdc:C00944', /global
NCDF_ATTPUT, fid, 'platform', 'NOAA-9 > National Oceanic & Atmospheric Administration-9; NOAA-10 > National Oceanic & Atmospheric Administration-10; NOAA-11 > National Oceanic & Atmospheric Administration-11; NOAA-12 > National Oceanic & Atmospheric Administration-12; NOAA-14 > National Oceanic & Atmospheric Administration-14; NOAA-15 > National Oceanic & Atmospheric Administration-15; NOAA-16 > National Oceanic & Atmospheric Administration-16; NOAA-18 > National Oceanic & Atmospheric Administration-18; NOAA-19 > National Oceanic & Atmospheric Administration-19; CHAMP > Challenging Minisatellite Payload, Gravity Recovery and Climate Experiment', /global
NCDF_ATTPUT, fid, 'sensor', 'GPS > Global Positioning System; MSU > Microwave Sounding Unit; AMSU > Advanced Microwave Sounding Unit', /global
NCDF_ATTPUT, fid, 'spatial_resolution', '2.5 x 2.5 degree', /global
NCDF_ATTPUT, fid, 'processing_level', 'NOAA Level 3', /global
NCDF_CONTROL, fid, /ENDEF

; Write values to the outfile
;-----------------------
NCDF_VARPUT, fid, vid_time, time
NCDF_VARPUT, fid, vid_climatology_bounds, climatology_bounds
NCDF_VARPUT, fid, vid_lat, lat
NCDF_VARPUT, fid, vid_lon, lon
NCDF_VARPUT, fid, vid_lat_bounds, lat_bounds
NCDF_VARPUT, fid, vid_lon_bounds, lon_bounds
NCDF_VARPUT, fid, vid_BT, BT_climatology
NCDF_CLOSE, fid

;
; Output Brightness Temperature Anomaly
;--------------------------------
; Time values
;------------------
time_bounds = fltarr(2,njdm)
time_bounds[0,*] = julday(month,1,year)-julday(1,1,1970)
time_bounds[1,*] = julday(month+1,0,year)-julday(1,1,1970)
time = total(time_bounds,1)/2.
ym_start = string(year[0],f='(i4.4)')+string(month[0],f='(i2.2)')
ym_end = string(year[njdm-1],f='(i4.4)')+string(month[njdm-1],f='(i2.2)')

; File name and dimensions
;------------------
Outfile = 'MLT-TTS-UCAR_V02R00_ANOM_s'+ym_start+'_e'+ym_end+'_c'+ymd_create+'.nc'
fid = NCDF_CREATE(Outpath+Outfile,/CLOBBER)
dim = size(BT_monthly,/dimensions)
latitude_dim = NCDF_DIMDEF(fid, 'lat', dim[1])
longitude_dim = NCDF_DIMDEF(fid, 'lon', dim[0])
time_dim = NCDF_DIMDEF(fid, 'time', dim[2])
bnds_dim = NCDF_DIMDEF(fid, 'bnds', 2)

; Variables definition
;------------------
NCDF_CONTROL, fid, /FILL
vid_time = NCDF_VARDEF(fid, 'time', time_dim,/float)
NCDF_ATTPUT, fid, vid_time, 'units', 'days since 1970-01-01 00:00:00'
NCDF_ATTPUT, fid, vid_time, 'long_name', 'time'
NCDF_ATTPUT, fid, vid_time, 'standard_name', 'time'
NCDF_ATTPUT, fid, vid_time, 'axis', 'T'
NCDF_ATTPUT, fid, vid_time, 'bounds', 'time_bounds'
NCDF_ATTPUT, fid, vid_time, 'missing_value', -999.
vid_lon = NCDF_VARDEF(fid, 'lon', longitude_dim,/float)
NCDF_ATTPUT, fid, vid_lon, 'units', 'degrees_east'
NCDF_ATTPUT, fid, vid_lon, 'long_name', 'longitude'
NCDF_ATTPUT, fid, vid_lon, 'standard_name', 'longitude'
NCDF_ATTPUT, fid, vid_lon, 'axis', 'X'
NCDF_ATTPUT, fid, vid_lon, 'bounds', 'lon_bounds'
NCDF_ATTPUT, fid, vid_lon, 'missing_value', -999.
vid_lat = NCDF_VARDEF(fid, 'lat', latitude_dim,/float)
NCDF_ATTPUT, fid, vid_lat, 'units', 'degrees_north'
NCDF_ATTPUT, fid, vid_lat, 'long_name', 'latitude'
NCDF_ATTPUT, fid, vid_lat, 'standard_name', 'latitude'
NCDF_ATTPUT, fid, vid_lat, 'axis', 'Y'
NCDF_ATTPUT, fid, vid_lat, 'bounds', 'lat_bounds'
NCDF_ATTPUT, fid, vid_lat, 'missing_value', -999.
vid_time_bounds = NCDF_VARDEF(fid, 'time_bounds', [bnds_dim,time_dim],/float)
NCDF_ATTPUT, fid, vid_time_bounds, 'valid_min', 0.
NCDF_ATTPUT, fid, vid_time_bounds, 'valid_max', julday(12,31,year[njdm-1])-julday(1,1,1970)
NCDF_ATTPUT, fid, vid_time_bounds, 'coordinates', 'time'
vid_lon_bounds = NCDF_VARDEF(fid, 'lon_bounds', [bnds_dim,longitude_dim],/float)
NCDF_ATTPUT, fid, vid_lon_bounds, 'valid_min', -180.
NCDF_ATTPUT, fid, vid_lon_bounds, 'valid_max', 180.
NCDF_ATTPUT, fid, vid_lon_bounds, 'coordinates', 'lon'
vid_lat_bounds = NCDF_VARDEF(fid, 'lat_bounds', [bnds_dim,latitude_dim],/float)
NCDF_ATTPUT, fid, vid_lat_bounds, 'valid_min', -90.
NCDF_ATTPUT, fid, vid_lat_bounds, 'valid_max', 90.
NCDF_ATTPUT, fid, vid_lat_bounds, 'coordinates', 'lat'
vid_BT = NCDF_VARDEF(fid, 'bt_anomaly', [longitude_dim,latitude_dim,time_dim],/float)
NCDF_ATTPUT, fid, vid_BT, 'units', 'K'
NCDF_ATTPUT, fid, vid_BT, 'long_name', 'NOAA Climate Data Record of Mean Layer Temperature - UCAR (Upper Trop & Lower Strat) Anomaly'
NCDF_ATTPUT, fid, vid_BT, 'standard_name', 'brightness_temperature_anomaly'
NCDF_ATTPUT, fid, vid_BT, 'valid_min', -50.
NCDF_ATTPUT, fid, vid_BT, 'valid_max', 50.
NCDF_ATTPUT, fid, vid_BT, 'missing_value', -999.
NCDF_ATTPUT, fid, vid_BT, 'coordinates', 'time lat lon'

; Global Attributies
;--------------------
NCDF_ATTPUT, fid, 'Conventions', 'CF-1.6', /global
NCDF_ATTPUT, fid, 'title', 'NOAA Climate Data Record of Mean Layer Temperature - UCAR (Upper Trop & Lower Strat) Anomaly', /global
NCDF_ATTPUT, fid, 'product_version', 'v02r00', /global
NCDF_ATTPUT, fid, 'software_version_id', 'gov.noaa.ncdc:/svn-repos/rsad/CDR/RO_CAL_BT/tags/v02r00/', /global
NCDF_ATTPUT, fid, 'source', 'UCAR COSMIC Data Analysis and Archival Center version 2010.2640', /global
NCDF_ATTPUT, fid, 'Metadata_Conventions', 'CF-1.6, Unidata Dataset Discovery v1.0, NOAA CDR v2.0, GDS v2.0', /global
NCDF_ATTPUT, fid, 'standard_name_vocabulary', 'CF Standard Name Table (v29, 08 July 2015)', /global
NCDF_ATTPUT, fid, 'id', outfile, /global
NCDF_ATTPUT, fid, 'naming_authority', 'gov.noaa.ncdc', /global
NCDF_ATTPUT, fid, 'date_created', string(cyear,f='(i4.4)')+'-'+string(cmonth,f='(i2.2)')+'-'+string(cdom,f='(i2.2)')+'T'+$
                   string(chour,f='(i4.4)')+':'+string(cmin,f='(i2.2)')+':'+string(csec,f='(i2.2)')+'Z', /global
NCDF_ATTPUT, fid, 'license', 'No constraints on data access or use', /global
NCDF_ATTPUT, fid, 'summary', 'The product is the monthly anomaly of adjusted MSU channel 3 and AMSU channel 7 brightness temperatures. Calculated brightness temperatures from an AMSU forward model using temperature and moisture profiles from radiosonde observations which identified by GPSRO were used to calibrate the channel 7 measurements from polar orbiters to a common reference.', /global
NCDF_ATTPUT, fid, 'keywords', 'Earth Science > Spectral/Engineering > Microwave > Brightness Temperature', /global
NCDF_ATTPUT, fid, 'keywords_vocabulary', 'NASA Global Change Master Directory (GCMD) Earth Science Keywords, Version 8.1', /global
NCDF_ATTPUT, fid, 'cdm_data_type', 'Grid', /global
NCDF_ATTPUT, fid, 'institution', 'UCAR/COSMIC', /global
NCDF_ATTPUT, fid, 'geospatial_lat_min', min(lat), /global
NCDF_ATTPUT, fid, 'geospatial_lat_max', max(lat), /global
NCDF_ATTPUT, fid, 'geospatial_lon_min', min(lon), /global
NCDF_ATTPUT, fid, 'geospatial_lon_max', max(lon), /global
NCDF_ATTPUT, fid, 'geospatial_lat_units', 'degrees_north', /global
NCDF_ATTPUT, fid, 'geospatial_lon_units', 'degrees_east', /global
NCDF_ATTPUT, fid, 'geospatial_lat_resolution', lat_grid, /global
NCDF_ATTPUT, fid, 'geospatial_lon_resolution', lon_grid, /global
NCDF_ATTPUT, fid, 'time_coverage_start', string(year_start,f='(i4.4)')+'-'+string(month_start,f='(i2.2)')+'-01T00:00:00Z', /global
NCDF_ATTPUT, fid, 'time_coverage_end', string(year_end,f='(i4.4)')+'-'+string(month_end,f='(i2.2)')+'-'+$
                   string(julday(month_end+1,0,year_end)-julday(month_end,0,year_end),f='(i2.2)')+'T23:59:59Z', /global
NCDF_ATTPUT, fid, 'time_coverage_duration', 'P1Y', /global
NCDF_ATTPUT, fid, 'time_coverage_resolution', 'P1M', /global
NCDF_ATTPUT, fid, 'cdr_program', 'NOAA Climate Data Record Program for satellites', /global
NCDF_ATTPUT, fid, 'cdr_variable', 'brightness temperature', /global
NCDF_ATTPUT, fid, 'Metadata_Link', 'gov.noaa.ncdc:C00944', /global
NCDF_ATTPUT, fid, 'platform', 'NOAA-9 > National Oceanic & Atmospheric Administration-9; NOAA-10 > National Oceanic & Atmospheric Administration-10; NOAA-11 > National Oceanic & Atmospheric Administration-11; NOAA-12 > National Oceanic & Atmospheric Administration-12; NOAA-14 > National Oceanic & Atmospheric Administration-14; NOAA-15 > National Oceanic & Atmospheric Administration-15; NOAA-16 > National Oceanic & Atmospheric Administration-16; NOAA-18 > National Oceanic & Atmospheric Administration-18; NOAA-19 > National Oceanic & Atmospheric Administration-19; CHAMP > Challenging Minisatellite Payload, Gravity Recovery and Climate Experiment', /global
NCDF_ATTPUT, fid, 'sensor', 'GPS > Global Positioning System; MSU > Microwave Sounding Unit; AMSU > Advanced Microwave Sounding Unit', /global
NCDF_ATTPUT, fid, 'spatial_resolution', '2.5 x 2.5 degree', /global
NCDF_ATTPUT, fid, 'processing_level', 'NOAA Level 3', /global
NCDF_CONTROL, fid, /ENDEF

; Write values to the outfile
;-----------------------
NCDF_VARPUT, fid, vid_time, time
NCDF_VARPUT, fid, vid_lat, lat
NCDF_VARPUT, fid, vid_lon, lon
NCDF_VARPUT, fid, vid_time_bounds, time_bounds
NCDF_VARPUT, fid, vid_lat_bounds, lat_bounds
NCDF_VARPUT, fid, vid_lon_bounds, lon_bounds
NCDF_VARPUT, fid, vid_BT, BT_anomaly
NCDF_CLOSE, fid

stop
end
