pro match_noaa_amsumsu
;=====================================================================
;@***h*  MW_Sounder_FCDR/match_noaa_amsumsu.pro
;
; NAME
;        match_noaa_amsumsu.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION:
;              For the MSU or AMSU BT data, this program loops over the 
;              specified range of Julian days and years, and reads in 
;              AMSU or MSU BT, time and locations each day. The BTs are
;               matched are written to an NetCDF file for later use.
;
; EXTERNALS:
;              none.
;
; USAGE:
;              The RUN parameters specifying the data to process are set
;              directly below this header. The user needs to edit these
;              values to process the desired values and then execute the
;              program from within IDL.
;
; INPUTS:
;              NOAA AMSU data should be stored with the following directory
;              structure and file format:
;
;                $InPath_noaa/YYYYMM/YYYYDDD_msu_{noaa}_nc
;
; OUTPUTS:
;              Converted matched NOAA coefficients to
;              IDL format data:
;                $OutPath/{noaa1}_{noaa2}/match_{noaa1}_{noaa2}_YYYYMM_nc
;
; AUTHOR:
;              Liang Peng / Ben Ho (?)
;
; REVISIONS:
;              Liang Peng (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright
;              statement in the main directory
;
;@*****
;=====================================================================

;**************************************************************************
; Set RUN parameters here:
;        noaas = Mission pairs to process
;       JD_BEG = The begining Julian day to process
;       JD_END = The ending Julian day to process
;         type = Type of mission
;      cri_mm  = Time criteria
;     cri_dist = Distance criteria
;      cri_lza = Scope of satellite zenith
;       InPath = Path to MSU radiance data
;      OutPath = Path to output data
;
;              Path values should contain a trailing '/'
;**************************************************************************
noaas = 'noaa15/noaa19'
JD_BEG = julday(1,1,1978)
JD_END = julday(1,1,2006)
type = 'msu'
cri_mm  = 60*15. ;unit is second
cri_dist = 200; unit is km
cri_lza = 15.  ;; cri for satellite zenith
Inpath = '../Data/tmp/extract/msu_BT/'
outpath = '../Data/tmp/match_noaa_amsumsu/'
;*********************************************************
; End of RUN parameters
;*********************************************************

;make variable
noaas = strsplit(noaas,'/',/extract)
noaa1 = noaas[0]
noaa2 = noaas[1]
if type eq 'msu' then begin
  schan = string(findgen(4)+1,f='(i2.2)')
endif else begin
  schan = string(findgen(15)+1,f='(i2.2)')
endelse

;
; Set Julian days that will be dealwithed
;-----------------------------------------------------------------------
jdms = timegen(start=JD_BEG,final=JD_END,unit='month')

;
; Loop over the Julian days to process
;----------------------------------
for ijdm = 0,n_elements(jdms)-1 do begin
  jdm = jdms[ijdm]
  caldat,jdm,mm,dd,yyyy
  smm = string(mm,f='(i2.2)')
  syyyy = string(yyyy,f='(i2.2)')

  ;
  ; Out file name and out variables
  ;----------------------------------
  outdir = ''+noaa1+'_'+noaa2+'/'
  if ~file_test(outdir) then spawn,'mkdir -p '+outdir
  outfile = outdir+'match_'+noaa1+'_'+noaa2+'_'+str_jd(jdm,'yyyymm')+'_nc'
  if file_test(outfile) then goto,pro_end
  mvar = ['year','doy','sec_noaa1','sec_noaa2','lat_noaa1','lon_noaa1','lat_noaa2','lon_noaa2',$
	  'sza_noaa1','sza_noaa2','lza_noaa1','lza_noaa2','bt'+schan+'_noaa1','bt'+schan+'_noaa2']

  ;
  ; Set initial values
  ;----------------------------------
  num = 10000000; 
  for im = 0,n_elements(mvar)-1 do begin
    mv = mvar[im]
    rt=execute(mv+' = fltarr(num)')
  endfor

  ;
  ; Loop over the Julian days to process
  ;----------------------------------
  kk = -1.
  for jd = julday(mm,1,yyyy),julday(mm+1,0,yyyy) do begin

    ;
    ; Set input file name
    ;----------------------------------
    infile1 = Inpath+str_jd(jd,'yyyymm')+'/'+str_jd(jd,'yyyyddd')+'_'+type+'_'+noaa1+'_nc'
    infile2 = Inpath+str_jd(jd,'yyyymm')+'/'+str_jd(jd,'yyyyddd')+'_'+type+'_'+noaa2+'_nc'

    if ~(file_test(infile1) and file_test(infile2)) then goto,next_jd
    print,infile1

    ;
    ; Read in data
    ;----------------------------------
    var = ['year','doy','sec','lat','lon','sza','lza','bt'+schan]
    ncid = NCDF_OPEN(infile1,/NOWRITE)
    for iv = 0,n_elements(var)-1 do begin
      vd = NCDF_VARID(ncid, var[iv])
      rt=execute('NCDF_VARGET, ncid,  vd, '+var[iv]+'_n1')
    endfor
    NCDF_CLOSE, ncid

    ncid = NCDF_OPEN(infile2,/NOWRITE)
    for iv = 0,n_elements(var)-1 do begin
      vd = NCDF_VARID(ncid, var[iv])
      rt=execute('NCDF_VARGET, ncid,  vd, '+var[iv]+'_n2')
    endfor
    NCDF_CLOSE, ncid

    ; Change dim of time variables
    ;---------------------------------- 
    nscan = n_elements(lat_n1[*,0])
    ntime = n_elements(year_n1)
    year_n1_all = fltarr(nscan,ntime)
    doy_n1_all = fltarr(nscan,ntime)
    sec_n1_all = fltarr(nscan,ntime)
    year_n2_all = fltarr(nscan,ntime)
    doy_n2_all = fltarr(nscan,ntime)
    sec_n2_all = fltarr(nscan,ntime)
    if type eq 'msu' then begin
      lza_n1_all = fltarr(nscan,ntime)
      lza_n2_all = fltarr(nscan,ntime)
      spn = [-47.35,-37.88,-28.41,-18.94,-9.47,0,9.47,18.94,28.41,37.88,47.35]
    endif
    for iscan = 0,nscan-1 do begin
      year_n1_all[iscan,*] = year_n1
      doy_n1_all[iscan,*] = doy_n1
      sec_n1_all[iscan,*] = sec_n1
      year_n2_all[iscan,*] = year_n2
      doy_n2_all[iscan,*] = doy_n2
      sec_n2_all[iscan,*] = sec_n2
      if type eq 'msu' then begin
        radius=6371.03
        dtr=.01745329
        lza_n1_all[iscan,*] = (asin(sin(spn[iscan]*dtr)*(radius+alt)/radius))/dtr
        lza_n2_all[iscan,*] = (asin(sin(spn[iscan]*dtr)*(radius+alt)/radius))/dtr
      endif
    endfor
    year_n1 = year_n1_all
    doy_n1 = doy_n1_all
    sec_n1 = sec_n1_all
    year_n2 = year_n2_all
    doy_n2 = doy_n2_all
    sec_n2 = sec_n2_all

    ;
    ; Select data
    ;----------------------------------
    id1 = where(abs(lat_n1) gt 60 and abs(lza_n1) le cri_lza)
    id2 = where(abs(lat_n2) gt 60 and abs(lza_n2) le cri_lza)
    if id1[0] eq -1 or id2[0] eq -1 then goto,next_jd
    for iv = 0,n_elements(var)-1 do begin
      rt=execute(var[iv]+'_n1='+var[iv]+'_n1[id1]')
      rt=execute(var[iv]+'_n2='+var[iv]+'_n2[id2]')
    endfor

    for k1 = 0,n_elements(year_n1)-1 do begin
      ;
      ; Matching two data sets
      ;----------------------------------
      R_earth = 6371.004
      pi = 3.141592654D
      cos_distance = cos(lat_n1[k1]/180.*pi)*cos(lat_n2/180.*pi)*$
		     cos(lon_n1[k1]/180.*pi-lon_n2/180.*pi)+$
		     sin(lat_n1[k1]/180.*pi)*sin(lat_n2/180.*pi)
      distance = R_earth*acos(cos_distance)

      k2 = where(doy_n1[k1] eq doy_n2                 and $
		 abs(sec_n1[k1] - sec_n2) lt cri_mm   and $
		 distance lt cri_dist                 and $
		 abs(lza_n1[k1] - lza_n2) lt cri_lza)
      if k2[0] eq -1 then goto,next
      st = kk+1
      nd = kk+n_elements(k2)

      for im = 0,n_elements(mvar)-1 do begin
	mv = mvar[im]
	if strmatch(mv,'*1') then begin 
	  mm = strreplace(mv,'_noaa1','_n1')
	  rt=execute(mv+'[st:nd]='+mm+'[k1]')
	endif else if strmatch(mv,'*2') then begin
	  mm = strreplace(mv,'_noaa2','_n2')
	  rt=execute(mv+'[st:nd]='+mm+'[k2]')
	endif else begin
	  rt=execute(mv+'[st:nd]='+mv+'_n1[k1]')
	endelse
      endfor
    
      kk = nd
      next:
    endfor
    next_jd:
  endfor   ;loop of day

  ;
  ; Output data
  ; --------------------------
  if kk le 0 then goto,pro_end
  ptrarr,ptr_var,n_elements(mvar)
  for im = 0,n_elements(mvar)-1 do begin
    rt=execute('*ptr_var[im] = '+mvar[im]+'[0:kk]')
  endfor

  fid = NCDF_CREATE(outfile,/CLOBBER)
  time_dim = NCDF_DIMDEF(fid, 'time', n_elements(*ptr_var[0]))
  NCDF_CONTROL, fid, /FILL
  vid = lonarr(n_elements(mvar))
  for iv = 0,n_elements(mvar)-1 do begin
    type = size(*ptr_var[iv],/tname)
    rt=execute('vid[iv] = NCDF_VARDEF(fid, mvar[iv], time_dim,/'+type+')');
  endfor
  NCDF_CONTROL, fid, /ENDEF
  for iv = 0,n_elements(mvar)-1 do begin
    NCDF_VARPUT, fid, vid[iv], *ptr_var[iv]
  endfor
  NCDF_CLOSE, fid ; Close the netCDF file.
endfor

pro_end:
stop 
end 

