pro match_raob_noaa
;=====================================================================
;@***h*  MW_Sounder_FCDR/match_raob_noaa.pro
;
; NAME
;        match_raob_noaa.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For the given range of Julian days, read in AMSU data
;              extracted from the specified NOAA dataset and simulated
;              AMSU brightness temperatures from profiles of the 
;              specified RAOB intrument types.
;
;              Find colocated measurements satisfying the matchup
;              criteria and then save the matched values in an ASCCI
;              file for later use.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              NOAA AMSU data should be stored with the following directory
;              structure and file format:
;
;                $InPath_noaa/{noaa}/YYYY/amsu_{noaa}_YYYY.DDD_ch5-12.dat 
;
;              RAOB simulated AMSU data should be stored in the format:
;
;                $InPath_raob/YYYY/AMSU_raob_atmBt_YYYY.DDD.dat 
;  
; OUTPUTS:     
;              Matched AMSU values are stored in ASCinum1 files with the following
;              directry structure and file format:
;
;                $OutPath/{noaa}_raob/YYYY/match_{noaa}_raob_YYYY.DDD.dat
;
; AUTHOR:      
;              Xinjia Zhou 
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;                noaa = The name of the NOAA satellite to process.
;                           values = {'noaa15', 'noaa16', 'noaa18', 
;                                     'noaa19', 'noaam2'}
;   YEAR_BEG,JDAY_BEG = The begining year and Julian day to process
;   YEAR_END,JDAY_END = The ending year and Julian day to process
;   InPath_raob        = The root path to the AMSU Brigtness Temps from RAOB files.
;   InPath_noaa       = The root path to the NOAA AMSU Brightness Temps files.
;   OutPath           = The root path for the ASCinum1 files to be generated.
;
;                       Path values should contain a trailing '/'
;
; Matching criteria for colocating RAOB and NOAA data:
;
;        CRI_time = TIME Criteria
;        CRI_dist = DISTANCE Criteria
;        CRI_sang = SATELLITE VIEWING ANGLE Criteria
;
;**************************************************************************
       noaa='noaa15'
   YEAR_BEG=2001
   JDAY_BEG=1
   YEAR_END=2014
   JDAY_END=304
 InPath_raob='../Data/ASCII_tmp/raob_AMSU_Tbs/'
 InPath_noaa='../Data/ASCII_tmp/extract/'
    OutPath='../Data/ASCII_tmp/match/'
   CRI_time=7200. ;                    TIME Criteria units = sec
   CRI_dist=300   ;                DISTANCE Criteria units = km
   CRI_sang=15.   ; SATELLITE VIEWING ANGLE Criteria units = angle
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Loop over the years to process
;----------------------------------
for year_loop=YEAR_BEG,YEAR_END do begin

  ; Set the Julian day for the begining of each month,
  ; the number of days in the month, and the number 
  ; of Julian days for this year
  ;--------------------------------------------------
  if((year_loop eq 2004) or $
     (year_loop eq 2008) or $
     (year_loop eq 2012)    ) then begin
    data_begin=[1,32,61,92,122,153,183,214,245,275,306,336]
    data_longs=[31,29,31,30,31,30,31,31,30,31,30,31]
    NJdays=366
  endif else begin
    data_begin=[1,32,60,91,121,152,182,213,244,274,305,335]
    data_longs=[31,28,31,30,31,30,31,31,30,31,30,31]
    NJdays=365
  endelse

  ; Set the starting and ending Julian days
  ;------------------------------------------
  day_start=1
  day_end  =NJdays
  if(year_loop eq YEAR_BEG) then day_start=JDAY_BEG
  if(year_loop eq YEAR_END) then day_end  =JDAY_END
  
  ; Loop over the Julian days for this year
  ;-------------------------------------------
  for day_loop=day_start,day_end do begin
    ncount = 0L

    ; Open output file
    ;-----------------------
    outfile=OutPath+noaa+'_raob/'+string(year_loop,format='(i4.4)')     $
         +'/match_'+noaa+'_raob_'+string(year_loop,format='(i4.4)')+'.' $
                                    +string( day_loop,format='(i3.3)')+'.dat'
    err=file_test(OutPath,/DIRECTORY,/WRITE)
    if(err ne 1) then begin
     print,' OutPath does not exist or does not have write permission'
     print,' OutPath=',OutPath
     STOP
    endif
    outdir=file_dirname(outfile)
    err=file_test(outdir,/DIRECTORY,/WRITE)
    if(err ne 1 ) then begin
     file_mkdir,outdir
    endif
    openw,7,outfile,ERROR=err
    if(err ne 0) then begin
     print,' ERROR opening output file: err=',err
     print,' outfile=',outfile
     STOP
    endif

    ; Construct RAOB Tbs input file name
    ;  e.g. 'AMSU_raob_Bt_2008_260.dat'
    ;--------------------------------------
    infile_raob=InPath_raob+string(year_loop,format='(i4.4)')     $
          +'/AMSU_raob_Bt_'+string(year_loop,format='(i4.4)')+'.' $
                           +string( day_loop,format='(i3.3)')+'.dat'

    ; Open the RAOB dataset for this day and read in data
    ;---------------------------------------------------------
    openr,5,infile_raob,error=err
    if(err ne 0) then begin
      print,'there is no RAOB datafile in', day_loop
      close,5
      free_lun,5
      goto,NEXT_DAY
    endif 

    max_num=3000L
    year_raob =intarr(max_num)
    day_raob  =intarr(max_num)
    time_raob=fltarr(max_num) ;unit is hour
    lat_raob=fltarr(max_num)
    lon_raob=fltarr(max_num)
    bt_ch7_raob=fltarr(max_num)
    bt_ch8_raob=fltarr(max_num)
    bt_ch9_raob=fltarr(max_num)
    bt_ch10_raob=fltarr(max_num)
    bt_ch11_raob=fltarr(max_num)
    bt_ch12_raob=fltarr(max_num)
    cloud_index  =intarr(max_num)
    ratp  =intarr(max_num)
    corn  =intarr(max_num)
    sirc  =intarr(max_num)
    valid_max=intarr(6,max_num)
    percentage_chan=fltarr(6,max_num)
    tbs=fltarr(15)
    inum1=0L

    while ~eof(5) do begin
      readf,5, year,day,time, xlat,xlon, tbs,ratp1,corn1,sirc1, level_num
      data_1=fltarr(3,level_num)
      data_2=fltarr(20,100)
      readf,5,data_1
      readf,5,data_2

      ; calculate the number of the real ROAB observation in weighting function maximan height range 
      ; i.e. [0.8*max_height,1.25*max_height], 
      ; max_height is the height where weighting function be maximan
      ;------------------------------------------------------------------------------------------
      for chan_loop=7,12 do begin
        chan_wf=data_2(4+chan_loop,*)
        max_id = where(chan_wf eq max(chan_wf))
        pressure_in_max = data_2(1,max_id(0))
        valid_id=where(data_1(1,*) gt pressure_in_max*0.8 and data_1(1,*) lt pressure_in_max/0.8,valid_num)
       ;if valid_num ne 0 then valid_max(chan_loop-7,inum1)=1
        valid_max(chan_loop-7,inum1)=valid_num
      endfor

      ; calculate the weighting percetage covered by real ROAB observation
      ;------------------------------------------------------------------------------------------
      for chan_loop=7,12 do begin
        percentage_valid=0.
        percentage_all  =0.
        for lev_loop =0,99 do begin
          if(data_2(4,lev_loop) eq 1) then percentage_valid=percentage_valid+data_2(4+chan_loop,lev_loop) ;valid wf
          percentage_all=percentage_all+data_2(4+chan_loop,lev_loop)          ;total wf
        endfor
        percentage_chan(chan_loop-7,inum1)=float(percentage_valid)/float(percentage_all)
      endfor

      ; get the clear and cloud condition using relative humidity
      ;  1) clear : the valid range cover from 931 hPa to 300 hPa, and all of the relative humidity less than 87% 
      ;  2) cloud : the relative humidity in any level is more than 87% 
      ;  3) unknown :  the valid range don't cover from 931 hPa to 300 hPa
      ;------------------------------------------------------------------------------------------
      valid_cloud = where( data_2(4,*) eq 1 ,num_cloud) ;valid range

      if max(data_2(3,valid_cloud)) ge 87 then begin
            cloud_id = 1  ;cloud
      endif else if max(data_2(1,valid_cloud)) ge 931.524 and min(data_2(1,valid_cloud)) le 300 then begin ; valid range is enough
          if min(data_2(3,valid_cloud)) ge 0 then begin ; each valid level has valid relative humidity
              cloud_id = 0 ; clear
          endif else begin ; some valid levels have unvalid relative humidity
              valid_humidity = where(data_2(3,valid_cloud) , num_humidity)
              if max(data_2(1,valid_cloud(valid_humidity))) ge 931.524 or min(data_2(1,valid_cloud(valid_humidity))) le 300 then begin
                cloud_id = -1 ; unknown
              endif else begin
                cloud_id = 0 ; clear
              endelse
          endelse
      endif else  begin ; valid range is not enough
          cloud_id = -1 ; unknown
      endelse

      ; Extract the desired values
      ;------------------------------
      year_raob(inum1) = year
      day_raob(inum1) = day 
      time_raob(inum1)= time
      lat_raob(inum1)= xlat
      lon_raob(inum1)= xlon
      bt_ch7_raob(inum1) = tbs(6) ;chan_7
      bt_ch8_raob(inum1) = tbs(7) ;chan_8
      bt_ch9_raob(inum1) = tbs(8) ;chan_9
      bt_ch10_raob(inum1) = tbs(9) ;chan_10
      bt_ch11_raob(inum1) = tbs(10) ;chan_11
      bt_ch12_raob(inum1) = tbs(11) ;chan_12
      cloud_index(inum1) = cloud_id
      ratp(inum1) = ratp1
      corn(inum1) = corn1
      sirc(inum1) = sirc1
      inum1=inum1+1L
    endwhile

    ; trim the data
    ;------------------------------
    year_raob =year_raob(0:inum1-1)
    day_raob  =day_raob(0:inum1-1)
    time_raob=time_raob(0:inum1-1)
    lat_raob=lat_raob(0:inum1-1)
    lon_raob=lon_raob(0:inum1-1)
    bt_ch7_raob=bt_ch7_raob(0:inum1-1)
    bt_ch8_raob=bt_ch8_raob(0:inum1-1)
    bt_ch9_raob=bt_ch9_raob(0:inum1-1)
    bt_ch10_raob=bt_ch10_raob(0:inum1-1)
    bt_ch11_raob=bt_ch11_raob(0:inum1-1)
    bt_ch12_raob=bt_ch12_raob(0:inum1-1)
    cloud_index  =cloud_index(0:inum1-1)
    ratp  =ratp(0:inum1-1)
    corn  =corn(0:inum1-1)
    sirc  =sirc(0:inum1-1)
    valid_max=valid_max(*,0:inum1-1)
    percentage_chan=percentage_chan(*,0:inum1-1)
    close,5
    free_lun,5

    ; read in NOAA15 AMSU data 
    ;--------------------------------
    caldat,julday(1,day_loop,year_loop), month_loop, month_day,month_year
    infile_noaa=InPath_noaa+noaa+'/'+string( year_loop,format='(i4.4)')     $
                  +'/amsu_'+noaa+'_'+string( year_loop,format='(i4.4)')+'.' $
                                    +string(  day_loop,format='(i3.3)')+'.dat'     

    ; Open the AMSU dataset for this day and read in data
    ;---------------------------------------------------------
    openr,6,infile_noaa,error=err
    if(err ne 0) then begin
      print,'there is no noaa datafile in', day_loop
      close,6
      free_lun,6
      goto,NEXT_DAY
    endif else begin
      ; If there are not enough profiles, then goto the 
      ; next day, otherwise allocate space and read in data.
      ;------------------------------------------------------
      inum2=file_lines(infile_noaa)
      if(Inum2 eq 0) then begin
        print,'there is no noaa data in', day_loop
        goto,NEXT_DAY
      endif
      temp=fltarr(22,inum2)
      hh_noaa=fltarr(inum2)
      readf,6,temp
      close,6
      free_lun,6
    endelse

    ; Extract the desired values
    ;------------------------------
    day_noaa =temp[1,*]
    time_noaa=temp[2,*]  ;unit is second
    lat_noaa =temp[3,*]
    lon_noaa =temp[4,*]
    sza_noaa =temp[5,*]  ;solar zenith angle
    sat_noaa =temp[6,*]  ;satellite zenith angle , scaning back and forth + and -
    bt_ch7_noaa =temp[13,*] ; chan7
    bt_ch8_noaa =temp[14,*] ; chan8
    bt_ch9_noaa =temp[15,*] ; chan9
    bt_ch10_noaa=temp[16,*] ; chan10
    bt_ch11_noaa=temp[17,*] ; chan11
    bt_ch12_noaa=temp[18,*] ; chan12


    ; Loop over the number of RAOB values
    ;--------------------------------------
    daycount=0L
    for kk=0,(inum1-1) do begin ;follow the RAOB number

      ; Load indices where the matchup criteria are satisfied
      ;--------------------------------------------------------

      ;distance equation
      ;--------------------
      radius = 6371  ;earth mean radius, km
      cos_distance = cos(lat_raob[kk]/180.*!pi)*cos(lat_noaa/180.*!pi)*$
                     cos(lon_raob[kk]/180.*!pi-lon_noaa/180.*!pi)+$
                     sin(lat_raob[kk]/180.*!pi)*sin(lat_noaa/180.*!pi)

      ; cos_distance might larger than 1 or less than -1, 
      ; and lead arithmetic error: Floating illegal operand
      ;--------------------------------------------------------
      id_plus_1 =where(cos_distance gt 1, num_plus_1)
      id_minus_1=where(cos_distance lt -1, num_minus_1)
      if num_plus_1 gt 0  then cos_distance(id_plus_1)=1
      if num_minus_1 gt 0 then cos_distance(id_minus_1)=-1
      distance = radius*acos(cos_distance)


      id=where((    day_noaa                       eq day_loop) and $
               (abs(time_noaa-3600.*time_raob[kk]) lt CRI_time) and $
               (   distance                        lt CRI_dist) and $
               (abs(sat_noaa)                      le CRI_sang)     ,icount)
      
      ; Write matched results to output file
      ;------------------------------------------
      if(icount gt 0) then begin
        for k2=0,(icount-1) do begin

          ; Remove the overlap part of polar orbit data
          ; between two consecutive granules 
          ;--------------------------------------
          overlap_id = where( sza_noaa[id[k2]]  eq sza_noaa[id]  and $
                              sat_noaa[id[k2]]  eq sat_noaa[id]  and $
                              time_noaa[id[k2]] eq time_noaa[id]  $
                              , overlap_num)
          if overlap_num gt 1 and k2 gt overlap_id[0] then goto,jump_overlap

          ncount=ncount+1L
          daycount=daycount+1L
          printf,7,year_loop,day_loop,time_noaa[id[k2]]/3600.,time_raob[kk],$
                   lat_noaa[id[k2]],lon_noaa[id[k2]],$
                   lat_raob[kk],lon_raob[kk],$
                   cloud_index[kk],ratp[kk], corn[kk],sirc[kk],$
                   sza_noaa[id[k2]],sat_noaa[id[k2]],$
                   bt_ch7_noaa[ id[k2]],bt_ch7_raob[ kk],$
                   bt_ch8_noaa[ id[k2]],bt_ch8_raob[ kk],$
                   bt_ch9_noaa[ id[k2]],bt_ch9_raob[ kk],$
                   bt_ch10_noaa[id[k2]],bt_ch10_raob[kk],$
                   bt_ch11_noaa[id[k2]],bt_ch11_raob[kk],$
                   bt_ch12_noaa[id[k2]],bt_ch12_raob[kk],$
                   valid_max[*,kk],  percentage_chan[*,kk],$
          format='(i4,1x,i3,2(1x,f6.2),4(1x,f8.3),2x,4i5,2(1x,f6.2),12(1x,f7.2),6(1x,i2),6(1x,f8.3))'

          ; goto next noaa pixel
          ;--------------------
          jump_overlap:

        endfor  ; k2=0,(icount-1)
      endif    ;icount gt 0  
    endfor ; kk=0,(inum1-1) 
    print, 'Y=',year_loop,' D=',day_loop, $
           ' day_match,total matches:',daycount,ncount

    ; goto target for next day
    ;---------------------------
    NEXT_DAY:

    ; Close output file
    ;-------------------
    close,7
    free_lun,7
  endfor ; day_loop=day_start,day_end
endfor ; year_loop=YEAR_BEG,YEAR_END
;
; End Program
;---------------
end 
