pro merra_hdf2sav
;=====================================================================
;@***h*  MW_Sounder_FCDR/merra_hdf2sav.pro
;
; NAME
;        merra_hdf2sav.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION:
;              This program loops over the specified range of Julian days
;              and years, and reads in MERRA HDF file for the day. The data 
;              are shrinked and save to IDL data file for later use.
;
; EXTERNALS:
;              none.
;
; USAGE:
;              The RUN parameters specifying the data to process are set
;              directly below this header. The user needs to edit these
;              values to process the desired values and then execute the
;              program from within IDL.
;
; INPUTS:
;              MERRA HDF format data:
;                $InPath_coef/
;
; OUTPUTS:
;              Converted MERRA HDF data to IDL format data
;              IDL format data:
;                $OutPath/YYYYDDD.sav
;
; AUTHOR:
;              Liang Peng / Ben Ho (?)
;
; REVISIONS:
;              Liang Peng (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright
;              statement in the main directory
;
;@*****
;=====================================================================

;**************************************************************************
; Set RUN parameters here:
;       JD_BEG = The begining Julian day to process
;       JD_END = The ending Julian day to process
;       InPath = Path to MERRA HDF format data
;      OutPath = Path to output data
;
;              Path values should contain a trailing '/'
;**************************************************************************
 JD_BEG = julday(1,1,1978)
 JD_END = julday(1,1,2006)
 Inpath = '../Data/Input/merra/'
Outpath = '../Data/tmp/extract/merra_sav/'
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Set Julian days that will be dealwithed
; -------------------------------------------
jds = timegen(start=jd_beg,final=jd_end,unit='day')

;
; Set variables
; -------------------------------------------
variables = 'SLP,PS,PHIS,H,QV,T'
variables = strsplit(variables,',',/extract)

;
; Loop over the Julian days to process
;--------------------------------------------
for ijd = 0,n_elements(jds)-1 do begin

  ;
  ; Set time variables
  ; -------------------------------------------
  jd = jds[ijd]
  caldat,jd,month,dom,year
  doy = julday(month,dom,year)-julday(1,0,year)
  syear = string(year,format='(i4.4)')
  smonth = string(month,format='(i2.2)')
  sdom = string(dom,format='(i2.2)')
  sdoy = string(doy,format='(i3.3)')
  
  ;
  ; Set output file name
  ; -------------------------------------------
  savename = Outpath+syear+sdoy+'.sav'
  if file_test(savename) then goto,next_day

  ;
  ; Set input file name
  ; -------------------------------------------
  infile0 = Inpath+'*'+syear+smonth+sdom+'.hdf'
  infile = file_search(infile0)
  infile = infile[0]
  if ~file_test(infile) then goto,next_day

  ;
  ; Read in data
  ; -------------------------------------------
  print,infile
  fileID = HDF_SD_Start(infile, /read)
  for iv = 0,n_elements(variables)-1 do begin
    dataset_index = HDF_SD_NameToIndex(fileID, variables[iv])
    datasetID = HDF_SD_Select(fileID, dataset_index)
    rt=execute('HDF_SD_GetData, datasetID, '+variables[iv])
  endfor
  HDF_SD_end,fileID

  ;
  ; Select data
  ; -------------------------------------------
  ilon = findgen(288)
  ilat = findgen(144)
  itime = [0,2,4,6]

  H = H[ilon,ilat,*,[0,2,4,6]]
  PHIS = PHIS[ilon,ilat,*] & PHIS = PHIS[*,*,[0,2,4,6]]
  PS = PS[ilon,ilat,*] & PS = PS[*,*,[0,2,4,6]]
  SLP = SLP[ilon,ilat,*] & SLP = SLP[*,*,[0,2,4,6]]
  QV = QV[ilon,ilat,*,[0,2,4,6]]
  T = T[ilon,ilat,*,[0,2,4,6]]

  ;
  ; Save data
  ; -------------------------------------------
  savefile:
  save,filename=savename,H,PHIS,PS,SLP,QV,T
  next_day:
endfor

stop
end

