pro merra_sav2txt
;=====================================================================
;@***h*  MW_Sounder_FCDR/merra_sav2txt.pro
;
; NAME
;        merra_sav2txt.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION:
;              This program loops over the specified range of Julian days 
;              and years, and reads in MERRA IDL file for the day. The 
;              data are interpolated and save to ASCII data file for later use.
;
; EXTERNALS:
;              none.
;
; USAGE:
;              The RUN parameters specifying the data to process are set
;              directly below this header. The user needs to edit these
;              values to process the desired values and then execute the
;              program from within IDL.
;
; INPUTS:
;              MERRA IDL format data:
;                $InPath_coef/
;
; OUTPUTS:
;              Converted MERRA IDL data to ASCII format data
;              IDL format data:
;                $OutPath/YYYYDDD.sav
;
; AUTHOR:
;              Liang Peng / Ben Ho (?)
;
; REVISIONS:
;              Liang Peng (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright
;              statement in the main directory
;
;@*****
;=====================================================================

;**************************************************************************
; Set RUN parameters here:
;       JD_BEG = The begining Julian day to process
;       JD_END = The ending Julian day to process
;       InPath = Path to MERRA IDL format data
;      OutPath = Path to output ASCII data
;
;              Path values should contain a trailing '/'
;**************************************************************************
 JD_BEG = julday(1,1,1978)
 JD_END = julday(1,1,2006)
 Inpath = '../Data/Input/merra/'
 Inpath = '../Data/tmp/extract/merra_sav/'
outpath = '../Data/tmp/extract/merra_prf/'
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Set Julian days that will be dealwithed
; -------------------------------------------
jds = timegen(start=jd_beg,final=jd_end,unit='day')

;
; Read in some variables example.hdf
; -------------------------------------------
variables=['XDim','YDim','Height']
fileID = HDF_SD_Start(Inpath_hdf+'example.hdf', /read)
for iv = 0,n_elements(variables)-1 do begin
  dataset_index = HDF_SD_NameToIndex(fileID, variables[iv])
  datasetID = HDF_SD_Select(fileID, dataset_index)
  rt=execute('HDF_SD_GetData, datasetID, '+variables[iv])
endfor
HDF_SD_end,fileID
lons = XDim
lats = YDim
level = Height

;
; Loop over the Julian days to process
;--------------------------------------------
for ijd = 0,n_elements(jds)-1 do begin

  ;
  ; Set time variables
  ; -------------------------------------------
  jd = jds[ijd]
  caldat,jd,month,dom,year
  doy = julday(month,dom,year)-julday(1,0,year)
  syear = string(year,format='(i4.4)')
  smonth = string(month,format='(i2.2)')
  sdom = string(dom,format='(i2.2)')
  sdoy = string(doy,format='(i3.3)')

  ;
  ; Set resolution of Latitude and Lontitude
  ; ---------------------------------------------
  res = 4
  sr = string(res,format='(i1.1)')

  ;
  ; Set pressure levels
  ; ---------------------------------------------
  get100pres,pres100

  ;
  ; Set input and output file name
  ; ---------------------------------------------
  infile = Inpath_sav+syear+sdoy+'.sav'
  outfile = Outpath+syear+sdoy+'.txt'
  if file_test(outfile) then goto,next_day
  if ~file_test(infile) then goto,next_day
  close,3
  openw,3,outfile+'.test'
  print,outfile
  restore,infile

  ;
  ; Loop over hours
  ;--------------------------------------------
  for ih = 0,3 do begin
    print,ih*6
    ;
    ; Loop over Latitude
    ;--------------------------------------------
    for ilat = floor(res/2),143,res do begin
      ;
      ; Loop over Longitude
      ;--------------------------------------------
      for ilon = floor(res/2),287,res do begin

        ;
        ; Set data in profile
        ; -------------------------------------------
	lat = lats[ilat]
	lon = lons[ilon]
	temp = t[ilon,ilat,*,ih]
	qvi = qv[ilon,ilat,*,ih]
	pres = level
	idnan = where(qvi gt 1)
	if idnan[0] gt -1 then qvi[idnan] = 0
	pw = qvi*pres/((qvi*(-0.378))+0.622)

        ;
        ; Interpolate data
        ; -------------------------------------------
	lpw = alog(pw
	lpres = alog(pres)
	lpres100 = alog(pres100)
	qv100 = interpol(qvi,lpres,lpres100)
	t100 = interpol(temp,lpres,lpres100)
	lpw = alog(pw)
	lpw100 = interpol(lpw,lpres,lpres100)
	pw100 = exp(lpw100)
	idt = where(t100 gt 400)
	if idt[0] gt -1 then begin
	  t100[idt] = 260
	endif

	idpw = where(~finite(pw100))
	if idpw[0] gt -1 then begin 
	  pw100[idpw] = 0
	endif

	vd0 = pres100*0+1
	id = where(pres100 gt max(pres) or pres100 lt min(pres))
	if id[0] gt -1 then vd0[id] = 0
	if idt[0] gt -1 then vd0[idt] = 0
	vd1 = vd0
	if idpw[0] gt -1 then vd1[idpw] = 0

        ;
        ; Outout data
        ;--------------------------------------------
	printf,3,year,doy,ih*6,lat,lon,format='(i4.4,1x,i3.3,1x,i2.1,1x,2(1x,f7.2))'
	for j = 0,99 do begin
	  printf,3,j,pres100[j],t100[j],1,pw100[j],1,$
	     format='(i4.1,2x,f9.4,1x,f8.4,2x,i1.1,2x,f9.4,2x,i1.1)'
	endfor

      endfor ; lon
    endfor ; lat
  endfor ; hour
  close,3
  spawn,'mv '+outfile+'.test '+outfile
  next_day:
endfor ; jd

pro_end:
stop
end

pro get100pres,pres100

pres100 = [0.0050,    0.0161,    0.0384,    0.0769,    0.1370, $
           0.2244,    0.3454,    0.5064,    0.7140,    0.9753,    1.2972, $
           1.6872,    2.1526,    2.7009,    3.3398,    4.0770,    4.9204,$
           5.8776,    6.9567,    8.1655,    9.5119,   11.0038,   12.6492,$
          14.4559,   16.4318,   18.5847,   20.9224,   23.4526,   26.1829,$
          29.1210,   32.2744,   35.6505,   39.2566,   43.1001,   47.1882,$
          51.5278,   56.1260,   60.9895,   66.1253,   71.5398,   77.2396,$
          83.2310,   89.5204,   96.1138,  103.0172,  110.2366,  117.7775,$
         125.6456,  133.8462,  142.3848,  151.2664,  160.4959,  170.0784,$
         180.0183,  190.3203,  200.9887,  212.0277,  223.4415,  235.2338,$
         247.4085,  259.9691,  272.9191,  286.2617,  300.0000,  314.1369,$
         328.6753,  343.6176,  358.9665,  374.7241,  390.8926,  407.4738,$
         424.4698,  441.8819,  459.7118,  477.9607,  496.6298,  515.7200,$
         535.2322,  555.1669,  575.5248,  596.3062,  617.5112,  639.1398,$
         661.1920,  683.6673,  706.5654,  729.8857,  753.6275,  777.7897,$
         802.3714,  827.3713,  852.7880,  878.6201,  904.8659,  931.5236,$
         958.5911,  986.0666,  990.9476,  995.2319,  1008.170]
end
