pro monthly_product
;=====================================================================
;@***h*  MW_Sounder_FCDR/monthly_product.pro
;
; NAME
;        monthly_product.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For specified range of Julian days, this program read in 
;              the MSU daily calibrated BTs and generate the monthly 
;              gridded BTs written to an IDL data file for later use.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              Daily NOAA MSU BT data:
;                $InPath_msu/
;              Daily NOAA AMSU BT data:
;                $InPath_amsu/
;
; OUTPUTS:     
;              Converted NOAA AMSU values are written to the 
;              ASCCI file:
;                $OutPath/YYYYMM.sav
;
; AUTHOR:      
;              Liang Peng / Ben Ho (?)
;
; REVISIONS:   
;              Liang Peng (Formatted for CDR)
; 
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================

;**************************************************************************
; Set RUN parameters here:
;      JD_BEG  = The Begining Julian day to process;julday(month,dayofmonth,year)
;      JD_END  = The Ending Julian day to process
;  InPath_msu  = Path to coeffcients.
; InPath_amsu  = Path to grid BT data
;     OutPath  = Path to output data
;   noaas_msu  = The MSU polar orbiter to use
;  noaas_amsu  = The AMSU polar orbiter to use
;
;              Path values should contain a trailing '/'
;**************************************************************************
JD_BEG      = julday( 1, 1,1985);
JD_END      = julday(12,31,2014)
InPath_msu  = '../Data/tmp/sno/rebuild/daily/'
InPath_amsu = '../Data/tmp/linfit/daily/'
Outpath     = '../Data/tmp/monthly/'
noaas_msu   = ['noaa14','noaa12','noaa11','noaa10','noaa9','noaa8','noaa7','noaa6','tiros']
noaas_amsu  = ['noaa15','noaa16','noaa18','noaa19','noaam2']
;*********************************************************
; End of RUN parameters
;*********************************************************

; 
; Generate the monthly julian days
; ----------------------------------
jdms = timegen(start=JD_BEG,final=JD_END,unit='month')

; 
; Set names of all satellites
; ---------------------------------
noaas = [noaas_msu,noaas_amsu]

; 
; Loop months to do the calculation
; ---------------------------------
for ijdm = 0,n_elements(jdms)-1 do begin

  ;
  ; Set time variables
  ; -------------------------------
  jdm = jdms[ijdm]
  caldat,jdm,month,dom,year
  syear  = string(year, f='(i4.4)')
  smonth = string(month,f='(i2.2)')

  ;
  ; Set initial values of number and brightness temperatures
  ; -------------------------------
  num_month = 0.0D
  bt_month  = 0.0D

  ;
  ; Loop noaa satellites 
  ; ------------------------------
  for inoaa = 0,n_elements(noaas)-1 do begin

    ; Set name of mission
    ; ----------------------------
    noaa = noaas[inoaa]    

    ; Set path of input data
    ; ----------------------------
    if total(noaa eq noaas_msu) eq 1 then begin
      inpath = inpath_msu
    endif else begin
      inpath = inpath_amsu
    endelse

    ; Loop julian day 
    ; ----------------------------
    for jd = julday(month,1,year),julday(month+1,0,year) do begin

      ; Set time variable
      ; --------------------------
      doy = jd - julday(1,0,year)
      sdoy   = string(doy,  f='(i3.3)')    

      ; Set input file name
      ; --------------------------
      infile = Inpath+syear+smonth+'/'+syear+sdoy+'_'+noaa+'_nc'
      if ~file_test(infile) then goto,next_day
      print,infile

      ; Read in data
      ; --------------------------
      var = ['chn_bin','num']
      ncid = NCDF_OPEN(infile,/NOWRITE)
      for iv = 0,n_elements(var)-1 do begin
	vd = NCDF_VARID(ncid, var[iv])
	rt=execute('NCDF_VARGET, ncid,  vd, '+var[iv])
      endfor
      NCDF_CLOSE, ncid
      idfin = where(num gt 0 and chn_bin gt 180 and chn_bin le 250,complement=idnan)
      if idnan[0] gt -1 then begin 
        num[idnan] = 0
        chn_bin[idnan] = 0
      endif

      ; Accumulate the BT and number
      ; --------------------------
      num_month = num_month+num
      bt_month = bt_month+chn_bin*num
      next_day:
    endfor ; day
  endfor ; noaa
  if n_elements(num_month) eq 1 then goto,next_month

  ; Calculate the monthly BT
  ; -------------------------
  bt_month = bt_month/num_month
  idnan = where(num_month eq 0)
  if idnan[0] gt -1 then begin
    bt_month[idnan] = -999
  endif

  ; Smooth
  ;--------------------------
  gd = 1
  BT_month[*,0:2] = -999
  BT_month[*,69:71] = -999
  BT0 = BT_month
  BT2 = 0.0
  num2 = 0.0
  for klon = 0-gd,gd do begin
  for klat = 0-gd,gd do begin
    BT1 = shift(BT0,[klon,klat])
    if klat gt 0 then BT1[*,0:klat] = -999
    if klat lt 0 then BT1[*,71+klat:71] = -999
    BT1[*,0:2] = -999
    BT1[*,69:71] = -999
    BT2 = BT2 + BT1 * (BT1 gt 50)
    num2 = num2 + (BT1 gt 50)
  endfor
  endfor
  BT_month = BT2/num2
  idnan = where(num2 eq 0)
  if idnan[0] gt -1 then BT_month[idnan] = -999

  ; Open output file
  ;--------------------------
  output:
  outfile=OutPath+syear+smonth+'.sav'
  save,filename=outfile,bt_month,num_month

  next_month: 
endfor

; 
; Program End
; ----------------------------
stop
end
