pro sno_step3_cnadir_dbin_msu
;=====================================================================
;@***h*  MW_Sounder_FCDR/sno_step3_cnadir_dbin_msu.pro
; 
; NAME
;        sno_step3_cnadir_dbin_msu.pro
;  
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For specified range of Julian days and years, the MSU 
;              rebuilt BTs are read in and calibrated by MERRA simulated 
;              BTs. Then the BTs are gridded and written to an NetCDF 
;              data file for later use.
; 
; EXTERNALS:   
;              none
; 
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
; 
; INPUTS:      
;              Rebuilt NOAA brighness temperature:
;                $InPath_rebuild/
; 
;              MERRA simulated MSU brightness temperatures in differents satellite zenith angles
;                $Inpath_MSU_MERRA/
;
;              MERRA simulated AMSU brightness temperatures
;                $Inpath_AMSU_MERRA/
; 
; OUTPUTS:     
;              Gridded calibrated rebuilt MSU brightness temperature
;                $Outpath/
; 
; AUTHOR:      
;              Liang Peng / Ben Ho (?)
; 
; REVISIONS:   
;              Liang Peng (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;           JD_BEG = The begining Julian day to process
;           JD_END = The ending Julian day to process
;             chan = MSU channel to process
;             noaa = Mission name to process 
;   Inpath_rebuild = Path to rebuild pixel level data
; Inpath_MSU_MERRA = Path to MERRA simulated MSU BT
;Inpath_AMSU_MERRA = Path to MERRA simulated AMSU BT
;          OutPath = Path to output data
;
;              Path values should contain a trailing '/'
;**************************************************************************
	    JD_BEG = julday(1,1,2001)
	    JD_END = julday(1,1,2001)
	      chan = 3
	      noaa = 'noaa14'
 Inpath_rebuild    = '../Data/tmp/sno/rebuild/orbit/'
 Inpath_MSU_MERRA  = '../Data/tmp/simulate/merra_msu_grid/'
 Inpath_AMSU_MERRA = '../Data/tmp/simulate/merra_amsu_grid/'
 Outpath           = '../Data/tmp/sno/rebuild/grid/'
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Set Julian days that will be dealwithed
;-----------------------------------------------------------------------
jds = timegen(start=JD_BEG,final=JD_END,unit='day')

;
; Loop over the Julian days to process
;----------------------------------
for ijd = 0,n_elements(jds)-1 do begin

  ;
  ; Set the month and year
  ;----------------------------------
  jd = jds[ijd]
  caldat,jd,month_loop,dom_loop,year_loop
  doy_loop = julday(month_loop,dom_loop,year_loop)-julday(1,0,year_loop)
  print,'------------------------'
  print,year_loop,month_loop,dom_loop,noaa
  print,'------------------------'

  ;
  ; Set Output file
  ;----------------------------------
  outfile = Outpath+string(year_loop,f='(i4.4)')+string(month_loop,f='(i2.2)')+'/'+$
            string(year_loop,f='(i4.4)')+string(doy_loop,f='(i3.3)')+'_msu_'+noaa+'_rebuild_bin_nc'
  ;if file_test(outfile) then goto,next_day

  ;
  ; Set input file and read in data
  ;----------------------------------
  infile = Inpath_rebuild+string(year_loop,f='(i4.4)')+string(month_loop,f='(i2.2)')+'/'+$
            string(year_loop,f='(i4.4)')+string(doy_loop,f='(i3.3)')+'_msu_'+noaa+'_rebuild_nc'
  if ~file_test(infile) then goto,next_day
  var = ['year','doy','sec','alt','lat','lon','tb','spn','tbw']
  ncid = NCDF_OPEN(infile,/NOWRITE)
  for iv = 0,n_elements(var)-1 do begin
    vd = NCDF_VARID(ncid, var[iv])
    rt=execute('NCDF_VARGET, ncid,  vd, '+var[iv])
  endfor
  NCDF_CLOSE, ncid

  ;
  ; Set hour
  ;----------------------------------  
  jday = julday(1,0,year,0,0,0)+doy+sec/3600.d/24.
  hour = (jday-jd)*24.

  ;
  ; Limb correction and convert to AMSU
  ;----------------------------------
  ; Set pentad date
  ;----------------------------------
  if julday(12,31,year_loop)-julday(1,0,year_loop) eq 365 then begin
    doy_pd = [1,    6,   11,   16,   21,   26,   31,   36,   41,   46,   51,   56,   61,   66,   71,   76,   81,   86,   $
             91,   96,  101,  106,  111,  116,  121,  126,  131,  136,  141,  146,  151,  156,  161,  166,  171,  176,   $
            181,  186,  191,  196,  201,  206,  211,  216,  221,  226,  231,  236,  241,  246,  251,  256,  261,  266,   $
            271,  276,  281,  286,  291,  296,  301,  306,  311,  316,  321,  326,  331,  336,  341,  346,  351,  356,  361]
  endif else begin
    doy_pd = [1,    6,   11,   16,   21,   26,   31,   36,   41,   46,   51,   56,   62,   67,   72,   77,   82,   87,   $
             92,   97,  102,  107,  112,  117,  122,  127,  132,  137,  142,  147,  152,  157,  162,  167,  172,  177,   $
            182,  187,  192,  197,  202,  207,  212,  217,  222,  227,  232,  237,  242,  247,  252,  257,  262,  267,   $  
            272,  277,  282,  287,  292,  297,  302,  307,  312,  317,  322,  327,  332,  337,  342,  347,  352,  357,  362]
  endelse

  ;
  ; Set day of simulated MERRA 
  ; ------------------------------------
  m_pd = min(abs(doy_loop-doy_pd),id_pd)

  ;
  ; Read in simulated MERRA AMSU
  ; ------------------------------------
  fn0 = Inpath_AMSU_MERRA+string(year_loop,f='(i4.4)')+string(doy_pd[id_pd],f='(i3.3)')+'.lza0_nc'
  ncid = NCDF_OPEN(fn0,/NOWRITE)
  vd = NCDF_VARID(ncid, 'bt')
  NCDF_VARGET, ncid,  vd, bt0
  NCDF_CLOSE, ncid
  bt_amsu = [[[bt0]],[[bt0[*,*,0]]]]

  ;
  ; Read in simulated MERRA MSU
  ; ------------------------------------
  rlza = [ 0,  10,  20,  30,  40,  50,  60 ]
  slza = ['0','10','20','30','40','50','60']
  latbin = findgen(36)*5.-90+2.5
  lonbin = findgen(72)*5.-180+2.5
  bt_msu = fltarr(72, 36, 5, n_elements(rlza))
  for ilza = 0,n_elements(rlza)-1 do begin
    fni = Inpath_MSU_MERRA+string(year_loop,f='(i4.4)')+string(doy_pd[id_pd],f='(i3.3)')+'.lza'+slza[ilza]+'_nc'
    ncid = NCDF_OPEN(fni,/NOWRITE)
    vd = NCDF_VARID(ncid, 'bt')
    NCDF_VARGET, ncid,  vd, bt
    NCDF_CLOSE, ncid
    bti = [[[bt]],[[bt[*,*,0]]]]
    bt_msu[*,*,*,ilza] = bti
  endfor
 
  ;  
  ; Convert scan angle to local zenith angle
  ; ------------------------------------
  radius=6371.03
  dtr=.01745329
  lza=(asin(sin(spn*dtr)*(radius+alt)/radius))/dtr

  ;
  ; Set interpolat values
  ; ------------------------------------
  ilat = (lat-latbin[0])/5.
  ilon = (lon-lonbin[0])/5.
  ihour = hour/6.
  ilza = abs(lza)/60.*(n_elements(rlza)-1)

  ;
  ; Set correction of brightness temperature
  ; ------------------------------------ 
  hour0 = 0-lon/360.*24
  idh = where(hour0 lt 0)
  if idh[0] gt -1 then hour0[idh] = hour0[idh]+24.
  ihour0 = hour0/6.

  bt_ref_amsu = interpolate(bt_amsu,ilon,ilat,ihour0)
  bt_ref_msu0 = fltarr(n_elements(lat),n_elements(rlza))
  for lza_loop = 0,n_elements(rlza)-1 do begin
    bt_ref_msu0[*,lza_loop] = interpolate(bt_msu[*,*,*,lza_loop], ilon,ilat,ihour)
  endfor
  bt_ref_msu = interpolate(bt_ref_msu0,findgen(n_elements(lat)),ilza)
  dbt = bt_ref_amsu-bt_ref_msu

  ;
  ; Accumulate values at each grid point. write the mean 
  ; values and number of data values to output file.
  ;------------------------------------------------------
  latbin = 2.5
  lonbin = 2.5
  ilat = fix((lat+90.0)/latbin)<(180./latbin-1)
  ilon = fix((lon+180.0)/lonbin)<(360./lonbin-1)
  ni =  ilon+ilat*144
  sni = sort(ni)
  ni = ni[sni]
  ui = uniq(ni)
  ni_ui = ni[ui]
  num = fltarr(144,72)
  if n_elements(ui) eq 1 then begin
    num[ni_ui] = n_elements(ni)
  endif else begin
    tnum = [ui[0]+1,ui[1:n_elements(ui)-1]-ui[0:n_elements(ui)-2]]
    num[ni_ui] = tnum
  endelse

  ptr_var = ptrarr(13,/allocate_heap)
  for id = 0,10 do begin
    sid = string(id,format='(i2.2)')
    cbt = tb[id,*]+dbt
    vl = double(cbt[sni])
    vl_csum  = total(vl,/CUMULATIVE)
    vl_csum_ui = vl_csum[ui]
    if n_elements(ui) eq 1 then begin
      vl_mean = total(vl)/n_elements(vl)
    endif else begin
      vl_sum = [vl_csum_ui[0],vl_csum_ui[1:n_elements(ui)-1]-vl_csum_ui[0:n_elements(ui)-2]]
      vl_mean = vl_sum/tnum
    endelse
    mval = dblarr(144,72)-999
    mval[ni_ui] = vl_mean
    *ptr_var[id] = mval
  endfor
  *ptr_var[11] = num
  vl = double(tbw[sni])
  vl_csum  = total(vl,/CUMULATIVE)
  vl_csum_ui = vl_csum[ui]
  if n_elements(ui) eq 1 then begin
    vl_mean = total(vl)/n_elements(vl)
  endif else begin
    vl_sum = [vl_csum_ui[0],vl_csum_ui[1:n_elements(ui)-1]-vl_csum_ui[0:n_elements(ui)-2]]
    vl_mean = vl_sum/tnum
  endelse
  mval = dblarr(144,72)-999
  mval[ni_ui] = vl_mean
  *ptr_var[12] = mval

  ;
  ; Write data to output file
  ;-------------------------------------
  var = 'tbbin'+string(findgen(11),format='(i2.2)')
  var = [var,'num','tbwbin']
  outdir = file_dirname(outfile)
  if ~file_test(outdir) then spawn,'mkdir -p '+outdir
  fid = NCDF_CREATE(outfile,/CLOBBER)
  lat_dim = NCDF_DIMDEF(fid, 'lat', 72)
  lon_dim = NCDF_DIMDEF(fid,'lon', 144)
  NCDF_CONTROL, fid, /FILL
  vid = lonarr(n_elements(var))
  for iv = 0,n_elements(var)-1 do begin
    type = size(*ptr_var[iv],/tname)
    rt=execute('vid[iv] = NCDF_VARDEF(fid, var[iv], [lon_dim,lat_dim],/'+type+')');
  endfor
  NCDF_CONTROL, fid, /ENDEF
  for iv = 0,n_elements(var)-1 do begin
    NCDF_VARPUT, fid, vid[iv], *ptr_var[iv]
  endfor
  NCDF_CLOSE, fid ; Close the netCDF file.

  next_day:
endfor
pro_end:
stop
end
