pro sno_step5_chrisycorr_msu
;=====================================================================
;@***h*  MW_Sounder_FCDR/sno_step5_chrisycorr_msu.pro
; 
; NAME
;        sno_step5_chrisycorr_msu.pro
; 
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For specified names of MSU mission pairs, this program read 
;              in the MSU intersatellite biases and bias between NOAA14 and 
;              radiosonde and generate the Christy coefficients written to 
;              IDL data file and ASCII file for later use.

; 
; EXTERNALS:   
;              none.
; 
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
; 
; INPUTS:      
;              Intersatellite bias:
;                $InPath_bias/
;
;              Calibration coefficients:
;                $Inpath_coef/
; 
; OUTPUTS:     
;              Christy calibration coefficients:
;                $OutPath/
; 
; AUTHOR:      
;              Liang Peng / Ben Ho (?)
; 
; REVISIONS:   
;              Liang Peng (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================

;**************************************************************************
; Set RUN parameters here:
;      JD_BEG  = The Begining Julian day to process;julday(month,dayofmonth,year)
;      JD_END  = The Ending Julian day to process
; InPath_coef  = Path to coeffcients.
; InPath_grid  = Path to grid BT data
; OutPath      = Path to output data
;     ilat     = latitude scope to calibration
;              Path values should contain a trailing '/'
;**************************************************************************
JD_BEG = julday( 1, 1,1979)
JD_END = julday(12,31,2006)
Inpath = '../Data/tmp/sno/rebuild/bias/'
Outpath = '../Data/tmp/sno/rebuild/coef/'
ilat = 0
inol = 4
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; string of latitude
; --------------------------------------
slat = 'lat'+string(ilat,format='(i2.2)')
snol = 'nol'+string(inol,format='(i2.2)')

time = timegen(start=JD_BEG,final=JD_END,units='day')

noaa2s = ['noaa14/noaa12','noaa12/noaa11','noaa11/noaa10']
names = ['noaa14','noaa12','noaa11','noaa10']
X = dblarr(n_elements(names)+n_elements(noaa2s)+1+n_elements(names),20000)
Y = dblarr(20000)
t1 = 0;

;
; Set X,Y between NOAA and RO
; -----i-------------------------------
;
; Read in bias of GPS and NOAA14
; ------------------------------------
restore,Inpath+'raob-noaa14_'+snol+'.sav'

;
; Set brightness temperature and warm target temperature
; ------------------------------------
tw02 = tw0_noaa[ilat]
tb1 = tb_raob[ilat,*]
tb2 = tb_noaa[ilat,*]
tw2 = tw_noaa[ilat,*]

; 
; Set time variables
; -----------------------------------
caldat,jds_bias,month,dom,year
doy = jds_bias-julday(1,0,year)
idoy = doy/(julday(12,31,year)-julday(1,0,year))

;
; Set inter satellite bias
; -----------------------------------
DT = bias[ilat,*] 
DT = DT[*]

;
; Select good data to process
; -----------------------------------
id = where(abs(DT) lt 10 and finite(tw2))
tb1 = tb1[id]
tb2 = tb2[id]
tw2 = tw2[id]
tw2 = tw2 - tw02
idoy = idoy[id]
DT = DT[id]
month = month[id]
jds = jds_bias[id]

;
; Set seasonal variation
; -----------------------------------
n2 = 'noaa14'
t2 = t1+n_elements(id)-1
mval = fltarr(12)
for imonth = 1,12 do begin
  id_month = where(month eq imonth)
  mval[imonth-1] = mean(DT[id_month])
endfor
mb = mval-mean(mval)
save,filename=Outpath+'season_bias_'+slat+'_'+snol+'.sav',mb

;
; Set X,Y for solue equation
; -----------------------------------
idnan = where(abs(DT-mean(DT)) gt 7)
if idnan[0] gt -1 then DT[idnan] = 999
Y[t1:t2] = Dt
id2 = where(names eq n2)
idc = n_elements(names)+n_elements(noaa2s)
it2 = where(names eq n2)+n_elements(names)+n_elements(noaa2s)+1
X[id2[0],t1:t2] = -tw2
X[idc[0],t1:t2] = 1
X[it2[0],t1:t2] = -(jds-julday(1,1,1980))

t1 = t2 + 1
tw01s = fltarr(n_elements(noaa2s))
tw02s = fltarr(n_elements(noaa2s))

;
; Get X,Y between NOAAs
; ---------------------------------------
for in = 0,n_elements(noaa2s)-1 do begin

  ;
  ; Read in intersatellite biases
  ; ------------------------------------
  noaas = strsplit(noaa2s[in],'/',/extract)
  n1 = noaas[0]
  n2 = noaas[1]
  restore,Inpath+n1+'-'+n2+'.sav'

  ; 
  ; Set time variables
  ; -----------------------------------
  caldat,jds_bias,month,dom,year
  doy = jds_bias-julday(1,0,year)
  idoy = doy/(julday(12,31,year)-julday(1,0,year))

  ; 
  ; Set brightness temperature and warm target temperature
  ; ------------------------------------
  tw01 = tw0_noaa1[ilat]
  tw02 = tw0_noaa2[ilat]
  tw01s[in] = tw01
  tw02s[in] = tw02

  tb1 = tb_noaa1[ilat,*,inol]
  tb2 = tb_noaa2[ilat,*,inol]
  tw1 = tw_noaa1[ilat,*]
  tw2 = tw_noaa2[ilat,*]

  ;
  ; Set inter satellite bias
  ; -----------------------------------
  DT = bias[ilat,*,inol]
  DT = DT[*]
  id = where(abs(DT) lt 100 and finite(tw1) and finite(tw2))

  ;
  ; Select good data to process
  ; -----------------------------------
  tb1 = tb1[id]
  tb2 = tb2[id]
  tw1 = tw1[id]
  tw2 = tw2[id]
  tw1 = tw1 - tw01
  tw2 = tw2 - tw02
  DT = DT[id]
  month = month[id]
  jds = jds_bias[id]

  ;
  ; Set X,Y for solue equation
  ; -----------------------------------
  noaa2 = strsplit(noaa2s[in],'/',/extract)
  n1 = noaa2[0]
  n2 = noaa2[1]
  t2 = t1+n_elements(id)-1
  idnan = where(abs(DT-mean(DT)) gt 0.5)
  if idnan[0] gt -1 then DT[idnan] = 999
  Y[t1:t2] = Dt
  id1 = where(names eq n1)
  id2 = where(names eq n2)
  idc = in+n_elements(names)
  it1 = where(names eq n1)+n_elements(names)+n_elements(noaa2s)+1
  it2 = where(names eq n2)+n_elements(names)+n_elements(noaa2s)+1

  X[id1[0],t1:t2] = tw1
  X[id2[0],t1:t2] = -tw2
  X[it1[0],t1:t2] =  (jds-julday(1,1,1980))
  X[it2[0],t1:t2] = -(jds-julday(1,1,1980))
  X[idc[0],t1:t2] = 1
  t1 = t2 + 1
endfor
rt:
X = X[*,0:t2]
Y = Y[0:t2]
id = where(abs(Y) le 100 )
Y = Y[id]
X = X[*,id]
rt = regress(X, Y, SIGMA=sigma, CONST=const)

;
; using the coeffients to remove the bias and generate the intersatellite bias
; ------------------------------------------------------------------
close,2
openw,2,Outpath+'christy_ro_coeffient_'+slat+'_'+snol+'.dat'
c1 = 0
n0 = strsplit(noaa2s[0],'/',/extract)
n0 = n0[0]
for in = 0,n_elements(noaa2s)-1 do begin
  noaa2 = strsplit(noaa2s[in],'/',/extract)
  n1 = noaa2[0]
  n2 = noaa2[1]

  id1 = where(names eq n1)
  id2 = where(names eq n2)
  idc = in+n_elements(names)
  it1 = where(names eq n1)+n_elements(names)+n_elements(noaa2s)+1
  it2 = where(names eq n2)+n_elements(names)+n_elements(noaa2s)+1

  a1 = rt[id1]
  a2 = rt[id2]
  t1 = rt[it1]
  t2 = rt[it2]

  if n1 eq n0 then c1 = rt[n_elements(names)+n_elements(noaa2s)]+const
  c2 = c1+rt[idc]+const
  printf,2,n1,tw01s[in],a1[0],c1,t1,f='(a10,5f20.10)'
  printf,2,n2,tw02s[in],a2[0],c2,t2,f='(a10,5f20.10)'
  c1 = c2
endfor
close,2

pro_end:
stop
end
