         program msu_merra
!=====================================================================
!@***h*  MW_Sounder_FCDR/msu_merra.f
! 
! NAME:        
!   msu_merra.exe
! 
! LOCATION:    
!   $SRC_DIR/Src_fortran/
!
! DESCRIPTION: 
!              For MERRA reanalysis data, this program loops over
!              the specified range of julian days and years, and reads
!              in all of the GPSRO profiles for each day.
!              Each profile is passed to an MSU forward model routine
!              which calculates the brightness temperatures for 4
!              MSU channels. The resulting brightness temperatures for
!              each day are written to an ASCII file for later use.
!
! EXTERNALS:   
!              MSUfast module for MSU Forward Model Calculations.
! 
! USAGE:       
!              For MERRA reanalysis profiles,
!              the user needs to first edit the RUN parameters for the program 
!              contained in the file 'CONFIG.msu_merra' to set the mission 
!              name, the range of days to process and the input/output paths 
!              for the data.
!
!              The program is then compiled to generate an executable:
!
!                   bash> msu_merra.make
!                   bash> vi CONFIG.msu_merra
!                   bash> ./msu_merra.exe
!
! PARAMETERS:
!                 YEAR_BEG = Begining year
!                 JDAY_BEG = Begining julian day
!                 YEAR_END = Ending year
!                 JDAY_END = Ending julian day
!                   InPath = Path to interpolated GPSRO profiles
!                  OutPath = Path to output brightness temperature files
!
! INPUTS:      
!              Input ASCII files containing the profile data should be 
!              stored with the following directory structure and file format:
!
!                    $InPath/YYYYDDD.dat
!
! OUTPUTS:     
!              AMSU brightness temperatures are stored in the files:
!
!                    $OutPath/YYYYDDD.dat
!
! AUTHOR:      
!              ??
!
! REVISIONS:   
!              Patrick Callaghan 
!               *Cleaned up and Packaged the AMSU Forward model routines 
!                 into module AMSUfast_mod.f for CDR
!               *Cleaned up, added program inputs via CONFIG file, and 
!                 Formatted main program for CDR
!@*****
!=====================================================================
        parameter (len=450,nlev=100,nlev42=42)
        dimension BUF(len)  
        real profile_number,xlat,xlon
        integer ncase,icheck  
        integer year, month, day, hour, minute         
        integer second 
        real surface_press,surface_temp,emiss_mw
        dimension ta(nlev),wa(nlev),pres(nlev)
        dimension tai(nlev),wai(nlev),oai(nlev),pai(nlev)

        dimension stdta(nlev),stdwa(nlev)
        dimension inum1(nlev),inum2(nlev)
        dimension sta(nlev),swa(nlev)
        dimension ps(nlev),ref(nlev)
        dimension ps42(nlev42),psi42(nlev42)
        dimension ta42(nlev42),wa42(nlev42) 
        dimension tai42(nlev42),wai42(nlev42) 

        real ta_ind(100),wa_ind(100),oa_ind(100)
        real ta_dep(100),wa_dep(100),oa_dep(100)
        real t_ind(301), t_dep(301)
        integer i,j,k,kanal,ll
        real taut(100)

        character*100 outfile,infile,arg1,arg2,arg3
        integer iout1,iout2 
        integer iout3,LENO 
        integer icase,ilev,sfc_lev,sfc_lev42
        integer inum(nlev)
        real msu_tb(4)
        real T_500_300,theta
        integer nn
        integer year_loop,day_loop,daystart,dayend,Njday,itheta
        character*4  year_str
        character*3  day_str
        character*10 theta_str(7),stheta
        integer len_theta

        ! Program Inputs
        !------------------
        integer      YEAR_END,YEAR_BEG
        integer      JDAY_END,JDAY_BEG
        character*80 InPath,OutPath
        integer      InPlen,OutPlen
        namelist /inputs/YEAR_END,YEAR_BEG,
     &                   JDAY_END,JDAY_BEG,
     &                   InPath,OutPath,theta_str

        data ps/     0.0050,    0.0161,    0.0384,    0.0769,    0.1370,
     +    0.2244,    0.3454,    0.5064,    0.7140,    0.9753,    1.2972,
     +    1.6872,    2.1526,    2.7009,    3.3398,    4.0770,    4.9204,
     +    5.8776,    6.9567,    8.1655,    9.5119,   11.0038,   12.6492,
     +   14.4559,   16.4318,   18.5847,   20.9224,   23.4526,   26.1829,
     +   29.1210,   32.2744,   35.6505,   39.2566,   43.1001,   47.1882,
     +   51.5278,   56.1260,   60.9895,   66.1253,   71.5398,   77.2396,
     +   83.2310,   89.5204,   96.1138,  103.0172,  110.2366,  117.7775,
     +  125.6456,  133.8462,  142.3848,  151.2664,  160.4959,  170.0784,
     +  180.0183,  190.3203,  200.9887,  212.0277,  223.4415,  235.2338,
     +  247.4085,  259.9691,  272.9191,  286.2617,  300.0000,  314.1369,
     +  328.6753,  343.6176,  358.9665,  374.7241,  390.8926,  407.4738,
     +  424.4698,  441.8819,  459.7118,  477.9607,  496.6298,  515.7200,
     +  535.2322,  555.1669,  575.5248,  596.3062,  617.5112,  639.1398,
     +  661.1920,  683.6673,  706.5654,  729.8857,  753.6275,  777.7897,
     +  802.3714,  827.3713,  852.7880,  878.6201,  904.8659,  931.5236,
     +  958.5911,  986.0666,  990.9476,  995.2319,  1008.170/

        data ps42 / .1,.2,.5,1.,1.5,2.,3.,4.,5.,7.,10.,15.,20.,25.,30.,
     +  50.,60.,70.,85.,100.,115.,135.,150.,200.,250.,300.,350.,400.,
     +  430.,475.,500.,570.,620.,670.,700.,780.,850.,920.,950.,1000.,
     +  1025.,1050./

        do i=1,42 
          psi42(i) = ps42(42+1-i)
        enddo 
         
        ilev = 100
        sfc_lev = 100
        sfc_lev42 = 42
        ncase = 28 

        !----------------------------------------------------------------------------------------
        ! initialization
        !----------------------------------------------------------------------------------------
        ! Set default values
        !--------------------
        YEAR_BEG=9999
        YEAR_END=2013
        JDAY_BEG=9999
        JDAY_END=9999
        InPath='NOPATH'
        OutPath='NOPATH'

        ! Read input values from CONFIG file
        !-------------------------------------
        open(unit=14,file='CONFIG.msu_merra',status='old')
        read(14,inputs)
        close(14)

        ! Set string lengths, other fixed paramters
        !--------------------------------------------
        InPlen=len_trim(InPath)
        OutPlen=len_trim(OutPath)
        
        ! Loop over the years to process
        !---------------------------------
        do year_loop=YEAR_BEG,YEAR_END

          ! Set the number of Julian days for this year
          !---------------------------------------------
          if((year_loop-2000).eq.(4*((year_loop-2000)/4))) then
            Njday=366
          else
            Njday=365
          endif

          ! Set the range of Julian days for the current year
          !---------------------------------------------------
          day_start=1 
          day_end  =Njday
          if(year_loop.eq.YEAR_BEG) day_start=JDAY_BEG
          if(year_loop.eq.YEAR_END) day_end  =JDAY_END

          ! Loop over Julian days
          !------------------------
          do day_loop=day_start,day_end

            ! Open the Input/Output files
            !-----------------------------
            write(year_str,'(i4.4)') year_loop
            write(day_str ,'(i3.3)') day_loop

            ! Loop over solar zenith angle
            !------------------------
            do itheta = 1,7
              stheta = theta_str(itheta)
              read(stheta,*) theta
              len_theta = len_trim(stheta)
              infile=InPath(1:InPlen)
     &                   //year_str//day_str//'.txt'

              outfile=OutPath(1:OutPlen)//
     &               year_str//day_str//
     &               '.lza'//stheta(1:len_theta)//'.dat'
              print*,"inputfile=",infile
              print*,"outputfile=",outfile
              
              LUI = 15
              open(LUI,FILE=infile,STATUS='OLD',err=300)
         
              iout = 16
              open(iout,FILE= outfile,STATUS='unknown',err=300)
              
              do while (.true.)            
                read(LUI,*,end=121)year,day,time,xlat,xlon

                do ii = 1,100
                  read(LUI,*)lev,pres(ii),ta(ii),inum1(ii),wa(ii),
     &                       inum2(ii)
                enddo 

                do ii=1,100
                  if(wa(ii).eq.0)wa(ii)=1e-5
                enddo

                !-----------------------------------------------------------              
                ! Convert p=100 levels to p=42 levels
                !-----------------------------------------------------------  
                do i=1,100
                 pai(i) = pres(100+1-i)
                 tai(i) = ta(100+1-i)
                 wai(i) = wa(100+1-i)
                enddo

                call intupt(pai,tai,100,psi42,tai42,42)
                call intupt(pai,wai,100,psi42,wai42,42)
                
                do i=1,42 
                  ta42(i) = tai42(42+1-i)
                  wa42(i) = wai42(42+1-i) 
                enddo  

                surface_temp = ta(100)

                !-----------------------------------------------------------              
                ! Loop for channel MSU cahnnel from kanal=1-4  
                !-----------------------------------------------------------              
                do kanal = 1,4 
                  call transmsu42(ta42,wa42,theta,kanal,taut,nlev42)
                  sfc_lev42 = 42
                  call intrte_mws(taut,ta42,surface_temp,emiss_mw,
     +                            sfc_lev42,tbbd,tbbr,tbbs,tbbt)
                  msu_tb(kanal) = tbbt
                enddo  !channel
                write(iout,2001)year,day,time,xlat,xlon,
     +                          (msu_tb(k),k=1,4) 
2001            format(1x,i4,1x,i3,1x,f9.3,1x,2(1x,f7.2),
     +                 2x,4(f7.2,1x))

              enddo  ! all profiles in one file
121           close(LUI)              
              close(iout)              
300           continue
            enddo ! theta
          enddo ! doy
        enddo ! year

        stop 
        end 
